# Main Repository Readme #

This is the main repository. A short getting started guide is described below.

## Introduction ##

 *  Before you can download the code you must have a user at our GitLab instance
    and SSH correctly set up. When you are able to run `ssh git.orbitntnu.no` and
    be greeted with your username, you know it works. See
    the [wiki page](https://git.orbitntnu.no/orbit/orbit/wikis/User-creation-and-SSH) for details.
 *  For instructions on how to download and install the repositories and
    required tools as quickly as possible, read the
    [short installation guide](#installing-short-version). It will guide you
    through installing the tools, downloading the code, building it and running
    the hello world program on an actual OBC board. At this point you're all set
    for writing programs, but you should really read on to get a better understanding
    of our ecosystem and how we do things.
 *  Further information about cloning and building can be found at the
    [Cloning](https://git.orbitntnu.no/orbit/orbit/wikis/cloning)
    and [Building](https://git.orbitntnu.no/orbit/orbit/wikis/building) pages respectively.
 *  Further information about our use of git submodules can be found at
    [this page](https://git.orbitntnu.no/orbit/orbit/wikis/submodules). You should
    read this page prior to making any new repositiories, to keep things consistent.
 *  For further information on how to manage repositories, code, ownership,
    licencing etc., see the (TBA).
 *  Individual modules/projects/programs usually have their own documentation,
    check the appropriate repositories for those.
 *  The kernel is documented [in its own wiki](https://git.erlendjs.no/erlends-os/kernel/kernel/wikis/home).
    Information about APIs and core utilities can be found there. If you find
    redundant information regarding repository management, core rules etc. there
    keep in mind that those are intended for kernel development and not necessary
    the Orbit specific software projects.
 *  The programs that you are likely to work on are in the `programs` directory
    which are specific to Orbit operations. Core utilities are found in `bin` and
    miscellangeous programs are located in `usr`. These folders contain programs
    that are to be uploaded and run on the OBC. The `kernel` and `kmod` folders
    contain the kernel itself and kernel modules, which are flashed into the
    microcontroller iself and not the filesystem. Offline documentation is located
    in `docs` and tools are found in `util`.

More information can be found at [the wiki pages](https://git.orbitntnu.no/orbit/orbit/wikis/home).


## Installing (short version) ##

For installing on a fresh Linux Mint 19 Cinnamon machine, use the commands below.
Note that you must already have ssh set up for this to work. These instructions
should work on any Debian based distro, substitute the package manager commands
for other distros.

```bash
sudo apt install git make gcc-arm-none-eabi libnewlib-arm-none-eabi libstdc++-arm-none-eabi-newlib build-essential picocom
cd Documents
mkdir git
cd git
git clone --recurse-submodules git@git.orbitntnu.no:orbit/orbit.git
cd orbit
sudo cp -r util/big-endian/armv7-ar /usr/lib/gcc/arm-none-eabi/6.3.1/thumb/
./configure.sh
./buildkernel.sh
./builduser.sh
```

In order to update the kernel on the OBC via JTAG you will need to install
the Segger tools from the Segger website. This is not needed for user programs
so you may skip this step if your OBC already has it installed.
There is a Segger probe at taket you can use should you need it.

To install and run the hello world program, do as below. If you are not
in the dialout group you will need to run `picocom` and `make flash` with `sudo`
in order to access the serial device. Adding your user to the dialout group is
highly recommended so that you do not need sudo every time.

```bash
cd programs/helloworld
make
picocom -b 38400 --imap lfcrlf /dev/ttyUSB0
```

Now make certain that there is nothing running on the OBC and that the prompt
is clear, or the following will fail. Exit picocom with Ctrl+a Ctrl+q.

```bash
make flash
picocom -b 38400 --imap lfcrlf /dev/ttyUSB0
helloworld
```

You should now see the "Hello, World!" message. You may want to spend some time
experimenting with other programs on the OBC, check directory contents etc. to
familiarize yourself with the environment.
