#!/bin/bash

# Default config
configtouse=tms570

# If there is an argument supplied
if (( $# == 1 )); then
	configtouse=$1
fi

# Check for valid configs
validconfigs=($(ls kernel/kernel/config -Idefault -Ilive))

# Print valid configurations
echo "Configuration options detected:"

RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m'

foundconfig=no

for i in "${validconfigs[@]}"
do
	if [[ "$i" = "$configtouse" ]]; then
		echo -e " * ${GREEN}" $i "${NC}"
		foundconfig=yes
	else
		echo " * " $i
	fi
done

# Check and print the configuration used
if [[ "$foundconfig" = "yes" ]]; then
	echo "Using configuration" $configtouse
else
	echo -e "${RED}No such configuration${NC}"
	exit
fi

# Set the config symlink in the kernel
cd kernel/kernel/config
rm -f live
ln -i -s $configtouse live

# Set the config symlink in the fatfs module
cd ../../../kmod/fatfs/config
rm -f live
ln -i -s tms570/normal live
