Since the development of application software is separated from kernel development, a slightly different build process is necessary. This guide explains how to create a new user program from scratch.

## Prerequisites

Before starting this guide, it is expected that you have installed the necessary tools to build and upload the kernel to the microcontroller. Basic knowledge of _make_ and _git_ is necessary. If you are working with the "Orbit" satellite repositories, you must go through the main readme to make certain that you have all the existing repositories installed and that you are able to build them.

# Creating your own Program as a submodule

First, create a new repository in the `programs` folder by clicking the green "New project" button [here](https://git.orbitntnu.no/orbit/programs). Choose a fitting name and description. Choose `Public` for Visibility, mark the `Initialize repository with a README` checkbox and click "Create Project". You should now see the repository page with a `README.md` file. Copy the SSH url. It should look like
```
git@git.orbitntnu.no:orbit/programs/yourProjectName.git
```
Next, add this repository as a submodule to the main orbit repository. That is, navigate into the programs folder in a terminal and type
```
git submodule add git@git.orbitntnu.no:orbit/programs/yourProjectName.git
```
Where yourProjectName is your actual project name. This also automatically downloads the repository. Now you can start adding files and folders to your project. You should start by copy-pasting the contents of the `programs/helloworld` project, in particular:
 *  the `src` folder
 *  the `LICENSE` file
 *  the `makefile` file
 *  and one hidden git related file, `.gitignore`.

Remember to change the project name inside the makefile before you build it. This will be the name of the program on the OBC. To commit the changes you made, use git how you normally would, while being navigated inside your projects folder. This will push the changes to the submodule's repository while leaving the orbit repository unchanged.

In addition, navigate outside your projects folder and add+commit+push the addition of the new submodule to the orbit repository.

If you now navigate to the [programs folder on Gitlab](https://git.orbitntnu.no/orbit/orbit/tree/master/programs), you should see many folders looking like _program_ @ _some hexadecimal_. These are links to other git repositories, that are located somewhere else which were added as a submodule to the orbit repository. You should see your newly created repository among them.

To upload the program to the OBC, navigate to where the makefile of your program is located and type `make flash`. Make sure *not* to be connected via a serial terminal while flashing.

## Coding Hints

Some available functions are documented in the [kernel wiki](https://git.erlendjs.no/erlends-os/kernel/kernel/wikis/home) under _Application Binary Interface (ABI)_.
Some examples from there:
 *  To save data into files use open() and write() and close(). Files saved this way are saved on the flash chip. 
 *  To save things on the eeprom, use the eerpom driver in `orbit/lib/libsensor/`. Include `eeprom/24c256.h`.
 *  Saving things on SD card is a planned feature.

All chips currently available on the PCB have example code in the `orbit/usr/sensor` submodule.

