This article documents how to do debugging with a jTag probe and kDbg.

## Requirements ##

Before continuing you need to install the software for the jTag debug probe.
You can find a probe at taket. You also need to have KDBG installed.

## Installing GDB ##

Previously we used the GDB version supplied with the `arm-none-eabi-gdb` package.
Sadly this package has been discontinued in favour of `gdb-multiarch`, which
is broken. To get GDB working you need to build one with the correct target.
Download the source and configure it with:

```bash
./configure --target=arm-none-eabi --with-expat
```

Building may take a few minutes depending on your machine.

## Configure KDBG ##

Start KDBG and enter Settings -> Global Options. In the "How to invoke GDB"
field, enter the following, but change the path to your GDB install directory:

```
/home/erlendjs/Downloads/setup/programming/gdb/gdb-8.2/gdb/gdb --fullname --nx
```

## Preparing Your Program ##

Since we are debugging almost bare-metal the debugger does not know when your
program is starting, so it cannot halt at the beginning of your main function.
The easy fix is to require that you press a key to continue in the main function.
You will then have time to start the debugger. Add the following code to the
beginning of your `main()` and re-upload your program to the board:

```C
printf ("Press any key to continue\n");
char c;
while (read (0, &c, 1) != 1);
```

## Start The GDB Server ##

The jTag probe GDB server will reset the microcontroller when starting so this
is a good time to start it. Open a terminal in the `kernel/kernel` directory
and run `make server`.

## Get The Load Addresses ##

The microcontroller does not feature memory virtualization so we must manually
tell the debugger where our program is loaded. In order to get these, run
`sys showrelocs 1` before starting your program. When you start your program
the kernel will show the addresses. You must then put these in the `Makefile`
of your project and run `make debug` which will build files that the debugger
needs. **After you are done debugging, run `sys showrelocs` to turn the messages off.**
Otherwise the `make flash` target will start failing. Your makefile should
have something like this in it with different addresses that you have provided:

```
NOREL_BASE_TEXT   = 0x2000E000
NOREL_BASE_RODATA = 0x20010800
NOREL_BASE_DATA   = 0x20010C00
NOREL_BASE_BSS    = 0x20011034
```

**You will have to do this step again every time you change your program** and
you need to debug it. This is because code changes will change the size of the
program and therefore the load addresses. If you are running multiple programs
at the same time it will also depend on these programs and the order they are
loaded in.

## Start The Debugging Session ##

With your program waiting at the `Press any key to continue` prompt, run
`make kdbg` in a terminal at your computer. If everything works you should see
a nice graphical debugger stopped at the beginning of your main function.
Note that when your program crashes or exits you should also restart the debugger
to get rid of any bad debugger state. **If the debug server glitches out** you
must restart that one also.
