# Git Submodules and why we use it #

From [GitHub](https://blog.github.com/2016-02-01-working-with-submodules/):
"Submodules allow you to include or embed one or more repositories as
a sub-folder inside another repository."

The main "Orbit" repository serves as the main repository linking the individual project
repositories using git submodules. By using submodules, we are able to pull
separate, individual projects (repositories) into one large "main" repository
and then only update that main repository when we are satisfied with our
actual project repositories. As a consequence the main repository will always
contain the last working version of everyone's projects so that we do not
interfere when working on separate things.

Depending on what you are working on, you may then check out specific versions
of your own or others code locally and then commit/branch etc. within those
repositories. As long as the main repository has not been touched, other users
will still see the old and hopefully working code. When you are ready to
finalize your changes, the main repository is updated so that its submodules
are checked out at the newest commit for other users as well.

At the end of the day, submodules allows us to:

 *  Have our own repository for our programs instead of just a subfolder with shared history
 *  Avoid git conflicts between unrelated software
 *  Keep unrelated software separate
 *  Enable separate issue tracking, merge requests, integration etc. for different softwares
 *  Use different contribution and code rules for different softwares. (In other words, you stand more freely to decide how things are done in your repository.)
 *  Entire repositories may be added or removed from the main repository at any time. This makes it possible to maintain a clean main repository while not having to make any of the older software inaccessible.

# How to use submodules #

## Initial Cloning ##

When you cloned the main repository you should have used the `--recurse-submodules` option as follows:

```bash
git clone --recurse-submodules git@git.orbitntnu.no:orbit/orbit.git
```

This extra option makes certain that all the sub-repositories (submodules) are properly initialized and downloaded.

## Adding a new repository as submodule ##

When you intend to make a new repository for your project, you must first create a repository for it on the Orbit GitLab server. Normally, all programs should be located in the [orbit/programs](https://git.orbitntnu.no/orbit/programs) repository group on the server. Use the "new project" button. Creating a readme for the project right away using the web interface is highly recommended so that the repository is not empty.

Once created, you must add the repository as a submodule to the main repository so that others will see it. Navigate to the Orbit repository and open a terminal in the `programs` folder, where your program will be added. Use the following command to add the repository, substituting `url` with the actual clone url of your repository:

```bash
git submodule add url
```

Git will then download the contents into a new folder with the same name as your repository. In order to push the addition of a new module, perform a `git push` from the main repository.

## Day to day operation ##

Normally you will only change files in specific submodules. In order to have git work with the files, navigate to the submodule folder before running commands such as `git add`, `git commit`, `git push` etc.

When you run git commands in a submodule directory, git will only consider files in that submodule and only modify that sub-repository. Once done you may change to another sub-repository or jump up to the main repository where you may then push the updated submodules upstream, so that others will see the updated submodules next time they pull changes.

## Pulling all changes from all submodules ##

If you want to pull for changes to a submodule, use the command `git submodule update` as follows:

```bash
git submodule update --remote programs/helloworld
```

The above example will update the hello world program to the version specified in the main repository, and also pull for changes from the server.

In some cases it may be necessary to pull all changes to all the submodules in one operation. From the main Orbit repository folder, you may do so as follows:

```bash
git submodule update --recursive --remote
```

The same command may also be used to check for changes in a set of submodules, such as only those found in `programs` or those in `bin`.