/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef ARCHCONFIG_H
#define ARCHCONFIG_H

///////////////////////////////////////////////////////////////////////////////
// Memory list settings
///////////////////////////////////////////////////////////////////////////////
#define MEM_INUSEBIT (1 << 31)
#define MEM_LISTSIZE 128

///////////////////////////////////////////////////////////////////////////////
// MPU Data
///////////////////////////////////////////////////////////////////////////////
#define PAGER_ENABLE 1
#define PAGER_MAX_ENTRIES 16		// Entries per page table block
#define PAGER_REGION_1 8			// First hardware region to use
#define PAGER_REGION_2 9			// Second hardware region to use
#define PAGER_MPUREGS_1 12			// Index of first block in ThreadControl
#define PAGER_MPUREGS_2 15			// Index of second block in ThreadControl

#define MPU_DO_EXTRA_PROCESS 1		// Extra processing required for arch

#if PAGER_ENABLE
#define MPU_REGS_NEEDED 3*6			// 4 Regions for program, 2 for pager
#else
#define MPU_REGS_NEEDED 3*4			// 4 Regions for program only
#endif

void PlatformMpuRegisterSetup (unsigned int *regs);
void PlatformPagerRegisterSetup (unsigned int *regs);

///////////////////////////////////////////////////////////////////////////////
// Loader check address
///////////////////////////////////////////////////////////////////////////////
#define LOADER_ADDRCHECK 0x08000000

///////////////////////////////////////////////////////////////////////////////
// Stack accessor macro
///////////////////////////////////////////////////////////////////////////////
#define ARCH_STACK_ACCESSOR(trc) \
	(((unsigned int) (trc)->stacktop) + 8)

#endif
