/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef NETWORK_CONFIG
#define NETWORK_CONFIG

///////////////////////////////////////////////////////////////////////////////
// Address configuration
///////////////////////////////////////////////////////////////////////////////

// The "MAC" address of this device
#define LAYER2_ADDRESS 0x29

// Default gateway, normal notation 192.1
#define IP_DEFAULT_GATEWAY 0xC001

// Default address of this node
#define IP_DEFAULT_ADDRESS 6

///////////////////////////////////////////////////////////////////////////////
// Interface init
///////////////////////////////////////////////////////////////////////////////
#define NET_INIT \
	static Layer2KTTY interface0; \
	KTTYInit (&interface0, "tty1"); \
	AddInterface ((Layer2*) &interface0);

///////////////////////////////////////////////////////////////////////////////
// Stack component configuration
///////////////////////////////////////////////////////////////////////////////

// Address Resolution Protocol
#define ARP_TIMEOUT 1000
#define ARP_USE_TABLE 1
#define ARP_TABLE_SIZE 5

// Allocation component
#define FRAMEPOOL_POOLSIZE 10
#define FRAMEPOOL_ERASE 0

// Driver component
#define RS485_TXFIFO_SIZE 10
#define RS485_RXFIFO_SIZE 10
#define RS485_RXBUFF_SIZE 512

// Link speed component
#define LAYER2LINKSPEED_USE_DEFAULT_SPEEDS 1

// Switch component
#define SWITCH_LISTSIZE 16
#define SWITCH_ENTRY_TTL 30

///////////////////////////////////////////////////////////////////////////////
// Special headers
///////////////////////////////////////////////////////////////////////////////

// Timing header to use
#define NETWORK_DELAY_INC "platform/platform.h"

///////////////////////////////////////////////////////////////////////////////
// Debug setup
///////////////////////////////////////////////////////////////////////////////

// Include required for the selected print function
#define NETWORK_DEBUG_INC "print/print.h"

// Print macros for strings and string+integer
#define NETWORK_PRINTS(str) Print(str)
#define NETWORK_PRINTI(str,i) Print(str,i)

// Per-module debug enable
#define NETWORK_DEBUG_ARP 0
#define NETWORK_DEBUG_FRAMEPOOL 0
#define NETWORK_DEBUG_LINKSPEED 0
#define NETWORK_DEBUG_INTERFACE 0
#define NETWORK_DEBUG_SWITCH 0
#define NETWORK_DEBUG_IP 0
#define NETWORK_DEBUG_OTHER 0

// Switch messages
#define DEBUG_SWITCH_ADDLIST 1
#define DEBUG_SWITCH_UPDATELIST 1
#define DEBUG_SWITCH_DELETELIST 1
#define DEBUG_SWITCH_BROADCASTING 1
#define DEBUG_SWITCH_ROUTELOOP 1

// Linkspeed
#define DEBUG_LINKSPEED_DISCOVERYSEND 1
#define DEBUG_LINKSPEED_STATEIS3 1
#define DEBUG_LINKSPEED_TESTFAILED 1
#define DEBUG_LINKSPEED_YIELDONDISCOVER 1
#define DEBUG_LINKSPEED_SENDINGLIST 1
#define DEBUG_LINKSPEED_YIELDONLISTREPLY 1
#define DEBUG_LINKSPEED_SENDINGSETSPEED 1
#define DEBUG_LINKSPEED_SETTINGSPEED 1
#define DEBUG_LINKSPEED_TEST 1
#define DEBUG_LINKSPEED_TESTREPLY 1
#define DEBUG_LINKSPEED_SPEEDCONFIRMED 1
#define DEBUG_LINKSPEED_TIMEOUT 1

#endif

