/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UART_STM32_H
#define UART_STM32_H

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Hardware
#define OVERSAMPLING_16 (0 << 15)
#define OVERSAMPLING_8 (1 << 15)
#define MODE_TXRX (3 << 2)
#define CR1_RXNEIE (1 << 5)
#define CR1_UE (1 << 13)

#define CR3_EIE (1 << 0)
#define CR3_IREN (1 << 1)
#define CR3_IRLP (1 << 2)
#define CR3_HDSEL (1 << 3)
#define CR3_NACK (1 << 4)
#define CR3_SCEN (1 << 5)
#define CR3_DMAR (1 << 6)
#define CR3_DMAT (1 << 7)
#define CR3_RTSE (1 << 8)
#define CR3_CTSE (1 << 9)
#define CR3_CTSIE (1 << 10)
#define CR3_ONEBIT (1 << 11)

// Default tty buffer size
#define TTY_BUF_SIZE 128
#define TTY_BUF_MASK 0x007F

// Number of special characters
#define NCCS 32

///////////////////////////////////////////////////////////////////////////////
// Registers
///////////////////////////////////////////////////////////////////////////////
typedef struct _UART
{
	volatile unsigned int SR;
	volatile unsigned int DR;
	volatile unsigned int BRR;
	volatile unsigned int CR1;
	volatile unsigned int CR2;
	volatile unsigned int CR3;
	volatile unsigned int GTPR;
} _UART;

#endif
