/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef PLATFORM_REG_TMS570_H
#define PLATFORM_REG_TMS570_H

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////

// 2.5.1 Primary System Control Registers (SYS)
typedef struct _SYS
{
	volatile unsigned int SYSPC1;
	volatile unsigned int SYSPC2;
	volatile unsigned int SYSPC3;
	volatile unsigned int SYSPC4;
	volatile unsigned int SYSPC5;
	volatile unsigned int SYSPC6;
	volatile unsigned int SYSPC7;
	volatile unsigned int SYSPC8;
	volatile unsigned int SYSPC9;
	volatile unsigned int r1;
	volatile unsigned int r2;
	volatile unsigned int r3;
	volatile unsigned int CSDIS;
	volatile unsigned int CSDISSET;
	volatile unsigned int CSDISCLR;
	volatile unsigned int CDDIS;
	volatile unsigned int CDDISSET;
	volatile unsigned int CDDISCLR;
	volatile unsigned int GHVSRC;
	volatile unsigned int VCLKASRC;
	volatile unsigned int RCLKSRC;
	volatile unsigned int CSVSTAT;
	volatile unsigned int MSTGCR;
	volatile unsigned int MINITGCR;
	volatile unsigned int MSINENA;
	volatile unsigned int MSTFAIL;
	volatile unsigned int MSTCGSTAT;
	volatile unsigned int MINISTAT;
	volatile unsigned int PLLCTL1;
	volatile unsigned int PLLCTL2;
	volatile unsigned int SYSPC10;
	volatile unsigned int DIEIDL;
	volatile unsigned int DIEIDH;
	volatile unsigned int r4;
	volatile unsigned int LPOMONCTL;
	volatile unsigned int CLKTEST;
	volatile unsigned int DFTCTRLREG;
	volatile unsigned int DFTCTRLREG2;
	volatile unsigned int r5;
	volatile unsigned int r6;
	volatile unsigned int GPREG1;
	volatile unsigned int r7;
	volatile unsigned int IMPFASTS;
	volatile unsigned int IMPFTADD;
	volatile unsigned int SSIR1;
	volatile unsigned int SSIR2;
	volatile unsigned int SSIR3;
	volatile unsigned int SSIR4;
	volatile unsigned int RAMGCR;
	volatile unsigned int BMMCR1;
	volatile unsigned int r8;
	volatile unsigned int CPURSTCR;
	volatile unsigned int CLKCNTL;
	volatile unsigned int ECPCNTL;
	volatile unsigned int r9;
	volatile unsigned int DEVCR1;
	volatile unsigned int SYSECR;
	volatile unsigned int SYSESR;
	volatile unsigned int SYSTASR;
	volatile unsigned int GLBSTAT;
	volatile unsigned int DEVID;
	volatile unsigned int SSIVEC;
	volatile unsigned int SSIF;
}_SYS;

// 2.5.3 Peripheral Central Resource (PCR) Control Registers
typedef struct _PCR
{
	volatile unsigned int PMPROTSET0;
	volatile unsigned int PMPROTSET1;
	volatile unsigned int r1[2];
	volatile unsigned int PMPROTCLR0;
	volatile unsigned int PMPROTCLR1;
	volatile unsigned int r2[2];
	volatile unsigned int PPROTSET0;
	volatile unsigned int PPROTSET1;
	volatile unsigned int PPROTSET2;
	volatile unsigned int PPROTSET3;
	volatile unsigned int r3[4];
	volatile unsigned int PPROTCLR0;
	volatile unsigned int PPROTCLR1;
	volatile unsigned int PPROTCLR2;
	volatile unsigned int PPROTCLR3;
	volatile unsigned int r4[4];
	volatile unsigned int PCSPWRDWNSET0;
	volatile unsigned int PCSPWRDWNSET1;
	volatile unsigned int r5[2];
	volatile unsigned int PCSPWRDWNCLR0;
	volatile unsigned int PCSPWRDWNCLR1;
	volatile unsigned int r6[2];
	volatile unsigned int PSPWRDWNSET0;
	volatile unsigned int PSPWRDWNSET1;
	volatile unsigned int PSPWRDWNSET2;
	volatile unsigned int PSPWRDWNSET3;
	volatile unsigned int r7[4];
	volatile unsigned int PSPWRDWNCLR0;
	volatile unsigned int PSPWRDWNCLR1;
	volatile unsigned int PSPWRDWNCLR2;
	volatile unsigned int PSPWRDWNCLR3;
}_PCR;

// Table 5-12. Flash Control Registers
typedef struct _FLASH
{
	volatile unsigned int FRDCNTL;
	volatile unsigned int r1;
	volatile unsigned int FEDACTRL1;
	volatile unsigned int FEDACTRL2;
	volatile unsigned int FCOR_ERR_CNT;
	volatile unsigned int FCOR_ERR_ADD;
	volatile unsigned int FCOR_ERR_POS;
	volatile unsigned int FEDACSTATUS;
	volatile unsigned int FUNC_ERR_ADD;
	volatile unsigned int FEDACSDIS;
	volatile unsigned int FPRIM_ADD_TAG;
	volatile unsigned int FDUP_ADD_TAG;
	volatile unsigned int FBPROT;
	volatile unsigned int FBSE;
	volatile unsigned int FBBUSY;
	volatile unsigned int FBAC;
	volatile unsigned int FBFALLBACK;

	volatile unsigned int FBPRDY;
	volatile unsigned int FPAC1;
	volatile unsigned int FPAC2;
	volatile unsigned int FMAC;
	volatile unsigned int FMSTAT;
	volatile unsigned int FEMU_DMSW;
	volatile unsigned int FEMU_DLSW;
	volatile unsigned int FEMU_ECC;
	volatile unsigned int r2;
	volatile unsigned int FEMU_ADDR;
	volatile unsigned int FDIAGCTRL;
	volatile unsigned int FRAW_DATAH;
	volatile unsigned int FRAW_DATAL;
	volatile unsigned int FRAW_ECC;
	volatile unsigned int FPAR_OVR;
	volatile unsigned int r3[16];
	volatile unsigned int FEDACSDIS2;
	volatile unsigned int r4[113];
	volatile unsigned int FSM_WR_ENA;
}_FLASH;

typedef struct _SYS2
{
	volatile unsigned int PLLCTL3;
	volatile unsigned int r1[14];
	volatile unsigned int CLK2CNTRL;
	volatile unsigned int VCLKACON1;
}_SYS2;

typedef struct _VIM
{
	volatile unsigned int IRQINDEX;
	volatile unsigned int FIQINDEX;
	volatile unsigned int r1[2];
	volatile unsigned int FIRQPR[4];
	volatile unsigned int INTREQ[4];
	volatile unsigned int REQENASET[4];
	volatile unsigned int REQENACLR[4];
	volatile unsigned int WAKEENASET[4];
	volatile unsigned int WAKEENACLR[4];
	volatile unsigned int IRQVECREG;
	volatile unsigned int FIQVECREG;
	volatile unsigned int CAPEVT;
	volatile unsigned int r2;
	volatile unsigned int CHANCTRL[32];
}_VIM;

typedef struct _VIMRAM
{
	void (*ptr[128])(void);
}_VIMRAM;

typedef struct _IOMM
{
	volatile unsigned int REVISION;
	volatile unsigned int r1[7];
	volatile unsigned int ENDIAN;
	volatile unsigned int r2[5];
	volatile unsigned int KICK[2];
	volatile unsigned int r3[40];
	volatile unsigned int ERR_RAW_STATUS;
	volatile unsigned int ERR_ENABLED_STATUS;
	volatile unsigned int ERR_ENABLE;
	volatile unsigned int ERR_ENABLE_CLR;
	volatile unsigned int r4;
	volatile unsigned int FAULT_ADDRESS;
	volatile unsigned int FAULT_STATUS;
	volatile unsigned int FAULT_CLEAR;
	volatile unsigned int r5[4];
	volatile unsigned int PINMMR[48];
}_IOMM;

typedef struct _RTI
{
	volatile unsigned int RTIGCTRL;
	volatile unsigned int RTITBCTRL;
	volatile unsigned int RTICAPCTRL;
	volatile unsigned int RTICOMPCTRL;
	volatile unsigned int RTIFRC0;
	volatile unsigned int RTIUC0;
	volatile unsigned int RTICPUC0;
	volatile unsigned int r1;
	volatile unsigned int RTICAFRC0;
	volatile unsigned int RTICAUC0;
	volatile unsigned int r2[2];
	volatile unsigned int RTIFRC1;
	volatile unsigned int RTIUC1;
	volatile unsigned int RTICPUC1;
	volatile unsigned int r3;
	volatile unsigned int RTICAFRC1;
	volatile unsigned int RTICAUC1;
	volatile unsigned int r4[2];
	volatile unsigned int RTICOMP0;
	volatile unsigned int RTIUDCP0;
	volatile unsigned int RTICOMP1;
	volatile unsigned int RTIUDCP1;
	volatile unsigned int RTICOMP2;
	volatile unsigned int RTIUDCP2;
	volatile unsigned int RTICOMP3;
	volatile unsigned int RTIUDCP3;
	volatile unsigned int RTITBLCOMP;
	volatile unsigned int RTITBHCOMP;
	volatile unsigned int r5[2];
	volatile unsigned int RTISETINTENA;
	volatile unsigned int RTICLEARINTENA;
	volatile unsigned int RTIINTFLAG;
	volatile unsigned int r6;
	volatile unsigned int RTIDWDCTRL;
	volatile unsigned int RTIDWDPRLD;
	volatile unsigned int RTIWDSTATUS;
	volatile unsigned int RTIWDKEY;
	volatile unsigned int RTIDWDCNTR;
	volatile unsigned int RTIWWDRXNCTRL;
	volatile unsigned int RTIWWDSIZECTRL;
	volatile unsigned int RTIINTCLRENABLE;
	volatile unsigned int RTICOMP0CLR;
	volatile unsigned int RTICOMP1CLR;
	volatile unsigned int RTICOMP2CLR;
	volatile unsigned int RTICOMP3CLR;
}_RTI;

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define SYS ((_SYS*)		0xFFFFFF00)
#define SYS2 ((_SYS2*)		0xFFFFE100)
#define PCR ((_PCR*)		0xFFFFE000)
#define FLASH ((_FLASH*)	0xFFF87000)
#define VIM ((_VIM*)		0xFFFFFE00)
#define VIMRAM ((_VIMRAM*)	0xFFF82000)
#define IOMM ((_IOMM*)		0xFFFFEA00)
#define RTI ((_RTI*)		0xFFFFFC00)

#endif
