/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DMA_H
#define DMA_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "arch/tms570ls1224/dma.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Initialize the DMA module. Gets it out of reset and sets its debug mode
 * to something appropriate. Should be called once on system startup.
 */
void DMAInitModule (void);

/*
 * Initialize a DMA channel and enable it for hardware requests.
 *   ch: The DMA channel (0 - 16)
 *   req: Request line, where the peripheral is connected (0 - 31)
 *   src: The source address. Note that you may have to add 3 because big endian.
 *   dst: Destination address.
 *   len: Length of the transfer in bytes.
 *   args: The value to be programmed into the CHCTRL register.
 */
void DMAChannelInit (int ch, int req,
	unsigned int src, unsigned int dst, unsigned int len,
	unsigned int args);

#endif
