/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef GPIO_H
#define GPIO_H

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define GPIO_INPUT GPIO_CNF_IN_FLOATING | GPIO_MODE_IN
#define GPIO_OUTPUT GPIO_CNF_OUT_GENERAL_PUSHPULL | GPIO_MODE_OUT_SPEED_HIGH
#define GPIO_AF GPIO_CNF_OUT_AF_PUSHPULL | GPIO_MODE_OUT_SPEED_HIGH
#define GPIO_ANALOG GPIO_CNF_IN_ANALOG | GPIO_MODE_IN

#define GPIO_CNF_IN_ANALOG (0 << 2)
#define GPIO_CNF_IN_FLOATING (1 << 2)
#define GPIO_CNF_IN_PULL (2 << 2)
#define GPIO_CNF_OUT_GENERAL_PUSHPULL (0 << 2)
#define GPIO_CNF_OUT_GENERAL_OPENDRAIN (1 << 2)
#define GPIO_CNF_OUT_AF_PUSHPULL (2 << 2)
#define GPIO_CNF_OUT_AF_OPENDRAIN (3 << 2)

#define GPIO_MODE_IN (0 << 0)
#define GPIO_MODE_OUT_SPEED_LOW (2 << 0)
#define GPIO_MODE_OUT_SPEED_MEDIUM (1 << 0)
#define GPIO_MODE_OUT_SPEED_HIGH (3 << 0)

#define PIN(num) (1 << num)

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct GPIO
{
	volatile unsigned int CRL;
	volatile unsigned int CRH;
	volatile unsigned int IDR;
	volatile unsigned int ODR;
	volatile unsigned int BSRR;
	volatile unsigned int BRR;
	volatile unsigned int LCKR;
} GPIO;

///////////////////////////////////////////////////////////////////////////////
// Registers
///////////////////////////////////////////////////////////////////////////////
#define GPIOA ((GPIO*) 0x40010800)
#define GPIOB ((GPIO*) 0x40010C00)
#define GPIOC ((GPIO*) 0x40011000)
#define GPIOD ((GPIO*) 0x40011400)
#define GPIOE ((GPIO*) 0x40011800)
#define GPIOF ((GPIO*) 0x40011C00)
#define GPIOG ((GPIO*) 0x40012000)

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void GPIOInit (GPIO *gpio, int pins, int mode);

#endif
