/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef GPIO_H
#define GPIO_H

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define GPIO_INPUT (0 << 0)
#define GPIO_OUTPUT (1 << 0)
#define GPIO_AF (2 << 0)
#define GPIO_ANALOG (3 << 0)

#define GPIO_PUSHPULL (0 << 4)
#define GPIO_OPENDRAIN (1 << 4)

#define GPIO_SPEED_LOW (0 << 8)
#define GPIO_SPEED_MEDIUM (1 << 8)
#define GPIO_SPEED_HIGH (2 << 8)
#define GPIO_SPEED_VERYHIGH (3 << 8)

#define GPIO_NOPULL (0 << 12)
#define GPIO_PULLUP (1 << 12)
#define GPIO_PULLDOWN (2 << 12)
#define GPIO_PULLBOTH (3 << 12)

#define GPIO_ALTERNATE(af) (af << 16)

#define PIN(num) (1 << num)

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct GPIO
{
	volatile unsigned int MODER;
	volatile unsigned int OTYPER;
	volatile unsigned int OSPEEDR;
	volatile unsigned int PUPDR;
	volatile unsigned int IDR;
	volatile unsigned int ODR;
	volatile unsigned int BSRR;
	volatile unsigned int LCKR;
	volatile unsigned int AFRL;
	volatile unsigned int AFRH;
} GPIO;

///////////////////////////////////////////////////////////////////////////////
// Registers
///////////////////////////////////////////////////////////////////////////////
#define GPIOA ((GPIO*) 0x40020000)
#define GPIOB ((GPIO*) 0x40020400)
#define GPIOC ((GPIO*) 0x40020800)
#define GPIOD ((GPIO*) 0x40020C00)
#define GPIOE ((GPIO*) 0x40021000)
#define GPIOF ((GPIO*) 0x40021400)
#define GPIOG ((GPIO*) 0x40021800)
#define GPIOH ((GPIO*) 0x40021C00)
#define GPIOI ((GPIO*) 0x40022000)
#define GPIOJ ((GPIO*) 0x40022400)
#define GPIOK ((GPIO*) 0x40022800)

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void GPIOInit (GPIO *gpio, int pins, int mode);

#endif
