/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef I2S_H
#define I2S_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "drivers/dma/dma-stm32f407.h"
#include "drivers/spi/spi-stm32f407.h"
#include "sched/scheduler.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Driver
#define AUDIO_BUFFER_BYTESIZE	4096 // In bytes
#define AUDIO_BUFFER_WORDSIZE	2048 // In 16-bit words
#define AUDIO_BUFFER_HALFSIZE	1024 // In 16-bit words

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct i2s
{
	// Pointer to hardware registers
	_SPI *instance;
	DMAHandle dmatx;

	// Configuration
	unsigned int config;
	unsigned int samplerate;

	// Other
	short dmaflag;
	short pidtonotify;
	struct ProcessControl *prctonotify;
	unsigned int audiopeaks;

	// Data buffer
	unsigned short audiodata[AUDIO_BUFFER_WORDSIZE];
} i2s;

#endif
