/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef LCD_H
#define LCD_H

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define LCD_VERTICAL 0
#define LCD_HORIZONTAL 1

#define TFT_BLACK 0x0000
#define TFT_WHITE 0xFFFF

#define TFT_RED 0xF800
#define TFT_GREEN 0x07E0
#define TFT_BLUE 0x001F

#define TFT_YELLOW 0xFFE0
#define TFT_MAGENTA 0xF81F
#define TFT_CYAN 0x07FF

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

// LCD Specific functions
void LCDInit (void);
void LCDRotate (int direction);
void LCDSetAddress (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);

// Helper function, does not interface LCD
unsigned int LCDColor (unsigned int r, unsigned int g, unsigned int b);

// Generic draw functions
void LCDText (char *str, unsigned int x, unsigned int y, int fontsize, unsigned int fcolor, unsigned int bcolor);
void LCDChar (char c, unsigned int x, unsigned int y, int fontsize, unsigned int fcolor, unsigned int bcolor);
void LCDRectangle (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int color);
void LCDVLine (unsigned int y1, unsigned int y2, unsigned int x, unsigned int color);
void LCDHLine (unsigned int x1, unsigned int x2, unsigned int y, unsigned int color);
void LCDLine (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int color);
void LCDDot (unsigned int x, unsigned int y, unsigned int color);
void LCDBox (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int color);
void LCDFill (unsigned int color);
void LCDData (unsigned short *data, int n);
void LCDGetMetrics (unsigned int *w, unsigned int *h);
int LCDGetOrientation (void);

#endif
