/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SPI_H
#define SPI_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "drivers/dma/dma-stm32f407.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Hardware
#define CR1_MSTR (1 << 2)
#define CR1_BRDIV2 (0 << 3)
#define CR1_BRDIV4 (1 << 3)
#define CR1_BRDIV8 (2 << 3)
#define CR1_BRDIV16 (3 << 3)
#define CR1_BRDIV32 (4 << 3)
#define CR1_BRDIV64 (5 << 3)
#define CR1_BRDIV128 (6 << 3)
#define CR1_BRDIV256 (7 << 3)
#define CR1_SPE (1 << 6)
#define CR1_SSI (1 << 8)
#define CR1_SSM (1 << 9)

///////////////////////////////////////////////////////////////////////////////
// Registers
///////////////////////////////////////////////////////////////////////////////
typedef struct _SPI
{
	volatile unsigned int CR1;
	volatile unsigned int CR2;
	volatile unsigned int SR;
	volatile unsigned int DR;
	volatile unsigned int CRCPR;
	volatile unsigned int RXCRCR;
	volatile unsigned int TXCRCR;
	volatile unsigned int I2SCFGR;
	volatile unsigned int I2SPR;
} _SPI;

#define SPI1 ((_SPI*) 0x40013000)
#define SPI2 ((_SPI*) 0x40003800)
#define SPI3 ((_SPI*) 0x40003C00)
#define SPI4 ((_SPI*) 0x40013400)
#define SPI5 ((_SPI*) 0x40015000)
#define SPI6 ((_SPI*) 0x40015400)

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct spi
{
	// Pointer to hardware registers
	_SPI *instance;
	DMAHandle dmatx;
	DMAHandle dmarx;

	// The index number of this instance
	unsigned int spinum;

	// If the spi is busy transferring data or not
	unsigned int busystate;
} spi;

#endif
