/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef TTY_H
#define TTY_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "../config/live/drivers/tty.h"
#include "arch/stm32f407/uart.h"
#include "drivers/dma/dma-stm32f407.h"

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct tty
{
	// Function pointers
	int (*read)(struct tty *t, void*, unsigned int);
	int (*write)(struct tty *t, void*, unsigned int);
	void (*irq)(struct tty *t);

	// Pointer to hardware registers
	_UART *instance;
	DMAHandle dmatx;
	DMAHandle dmarx;

	// The index number of this instance
	unsigned int ttynum;

	// Configuration of the UART
	unsigned int stopbits;
	unsigned int word_parity_mode_sampling;
	unsigned int flowcontrol;
	unsigned int baudrate;

	// If the tty is busy transferring data or not
	unsigned int busystate;

	// Read position
	unsigned int readpos;

	// Internal transfer buffers used for DMA
	unsigned char defrxbuf[TTY_BUF_SIZE];
	unsigned char deftxbuf[TTY_BUF_SIZE];
	unsigned char *rxbuf;
	unsigned int rxbufsize;

	// Position in the read buffer
	short pos_old;
	short irq_pos_old;

	// Process group id used for signals
	short pgid;

	// Error state and mode
	unsigned char errflag;
	unsigned char mode;
} tty;

struct termios
{
	// Standard stuff for the user
	unsigned int c_iflag;		// Input modes
	unsigned int c_oflag;		// Output modes
	unsigned int c_cflag;		// Control modes
	unsigned int c_lflag;		// Local modes
	unsigned char c_cc[NCCS];	// Special characters

	// Non-standard stuff
	unsigned int baudrate;
};

#endif
