/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef TTY_H
#define TTY_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "arch/tms570ls1224/tty.h"
#include "../config/live/drivers/tty.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Default tty buffer size
#define TTY_BUF_SIZE 128
#define TTY_BUF_MASK 0x007F

// Number of special characters
#define NCCS 32

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct tty
{
	// Pointer to hardware registers
	_SCI *instance;

	// The index number of this instance
	unsigned int ttynum;

	// Configured baudrate
	unsigned int baudrate;

	// If the tty is busy transferring data or not
	// 0 = ready, 1 = busy, 2 = error
	unsigned int busystate;

	// Read position
	unsigned int readpos;
	unsigned int readposirq;

	// Internal transfer and receive buffers used for DMA
	unsigned char defrxbuf[TTY_BUF_SIZE];
	unsigned char deftxbuf[TTY_BUF_SIZE];

	// Process group id used for signals
	short pgid;

	// DMA Information
	unsigned char reqchrx;
	unsigned char reqchtx;
	unsigned char dmachrx;
	unsigned char dmachtx;
	unsigned char dmachbs;

	// Function pointers
	int (*read)(struct tty *t, void*, unsigned int);
	int (*write)(struct tty *t, void*, unsigned int);
	void (*irq)(struct tty *t);
} tty;

struct termios
{
	// Standard stuff for the user
	unsigned int c_iflag;		// Input modes
	unsigned int c_oflag;		// Output modes
	unsigned int c_cflag;		// Control modes
	unsigned int c_lflag;		// Local modes
	unsigned char c_cc[NCCS];	// Special characters

	// Non-standard stuff
	unsigned int baudrate;
};

#endif
