/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"

///////////////////////////////////////////////////////////////////////////////
// Macros
///////////////////////////////////////////////////////////////////////////////

/*
 * Use this to add your driver to the /dev file system at compile time.
 */
#define MODULE_DESCRIPTOR(descriptorname,open,close,read,write,ioctl,name) \
	struct DeviceDriver dev_##descriptorname \
	__attribute((__section__(".dev_list"))) \
	__attribute((__used__)) \
	= {open, close, read, write, ioctl, name};

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////

/*
 * Device driver definition used by the device file system and device drivers.
 * It is recommended to keep this as a static struct in flash memory. Use
 * the MODULE_DESCRIPTOR to declare this structure, as it sets it up in the
 * proper section.
 */
struct DeviceDriver
{
	int (*open)(struct FileNode*, const char*);
	int (*close)(struct FileNode*);
	int (*read)(struct FileNode*, void*, unsigned int);
	int (*write)(struct FileNode*, void*, unsigned int);
	int (*ioctl)(struct FileNode*, unsigned int, void*);
	char *name;
};

///////////////////////////////////////////////////////////////////////////////
// Exports
///////////////////////////////////////////////////////////////////////////////
extern const struct VFSDriver vfs_driver_dev;
