/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "mem/string.h"
#include "mem/mem.h"
#include "posix/errno.h"

#include "sched/scheduler.h"
#include "process/processcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define PIPE_FLAG_INUSE	0x0001
#define PIPE_FLAG_NAMED	0x0002
#define PIPE_FLAG_MSG	0x0004
#define PIPE_FLAG_BLOCK	0x0008

#define PIPE_BUF_SIZE	512
#define PIPE_BUF_MASK	0x01FF
#define PIPE_NAME_MAXSIZE 256

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct pipe_data
{
	struct pipe_data *next;
	struct List blocked_readers;
	struct List blocked_writers;
	unsigned char contents[PIPE_BUF_SIZE];
	unsigned short readpos;
	unsigned short writepos;
	unsigned short flags;
	unsigned short nbytesinpipe;
	unsigned short usecounter_read;
	unsigned short usecounter_write;
	char name[PIPE_NAME_MAXSIZE];
}pipe_data;

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////
extern char vfs_interrupted;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

// pipe-alloc.c
pipe_data *vfs_pipe_allocate (pipe_data **root, const char *name);
void vfs_pipe_free (pipe_data **root, pipe_data *p);

// pipe-nonblocking.c
int vfs_pipe_read_nonblocking (pipe_data *p, void *data, int nbytes);
int vfs_pipe_write_nonblocking (pipe_data *p, void *data, int nbytes);

// pipe-blocking.c
int vfs_pipe_read_blocking (pipe_data *p, void *data, int nbytes);
int vfs_pipe_write_blocking (pipe_data *p, void *data, int nbytes);
void vfs_pipe_wake_eof (pipe_data *p);
