/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef MEM_H
#define MEM_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "../config/live/archconfig.h"

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
struct MemMpuReturn
{
	unsigned int dataptr;
	unsigned int mpu_rbar;
	unsigned int mpu_rasr;
};

struct MemTableEntry
{
	unsigned int addr;
	unsigned int nextaddr;
};

struct MemBank
{
	unsigned int bankaddress;
	unsigned int banksize;

	unsigned int currentfree;
	unsigned int lowestfree;
	unsigned int lastaddr;

	struct MemTableEntry memlist[MEM_LISTSIZE];
	unsigned char memilist[MEM_LISTSIZE];

	struct MemBank *nextbank;
};

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Initialize a bank "superblock". Do this BEFORE adding it to the memory
 * system.
 */
void MemBankInit (struct MemBank *b, unsigned int address, unsigned int size);

/*
 * Add a bank to the memory system. Note that the order tings are added
 * is important to later "bankno" arguments.
 */
void MemAddBank (struct MemBank *bank);

/*
 * Allocate memory from the given bank.
 */
unsigned int MemAlloc (unsigned int nbytes, unsigned int bankno);

/*
 * MPU Aware allocation from the given bank. The data pointer to use is the one
 * in the mpu structure, while the pointer that must be given to MemFree is the
 * one returned.
 */
unsigned int MemAllocMpu (unsigned int nbytes, unsigned int bankno, struct MemMpuReturn *mpu);

/*
 * Free a block of memory.
 */
int MemFree (unsigned int addr);

/*
 * Destructive test of a memory portion. Memory is divided into 16 blocks,
 * and each block is tested for 512 bytes at a time with 16-bit data.
 *
 * Returns zero if errors are detected.
 */
int MemTest (unsigned int addr, unsigned int nbytes);

#endif
