/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef MPU_H
#define MPU_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "elf/elf.h"
#include "process/processcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Definitions for region manipulation, follows the Cortex-M registers.
// Also works for Cortex-R by splitting the register correctly.
#define XN (1 << 28)
#define AP(x) ((x) << 24)
#define TEX(x) ((x) << 19)
#define S (1 << 18)
#define C (1 << 17)
#define B (1 << 16)
#define SIZE(x) ((x) << 1)
#define E 1

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Only to be called once on kernel startup.
 */
void MpuInitialSetup (void);

/*
 * MPU Aware allocation
 */
int MpuSpawnAllocate (struct ProcessControl *slot, struct Loader *loader,
	unsigned int argvsize, unsigned int envsize, unsigned int stacksize,
	struct Program *p);

#endif
