/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef PAGER_H
#define PAGER_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "process/processcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Only to be called once on kernel startup. It clears all table entries.
 */
void PagerInitialize (void);

/*
 * Loops through the table and frees any memory associated with tid,
 * also removing the entry in the table
 */
void PagerCleanupByPID (unsigned int tid);

/*
 * Add an entry to the page table.
 *  tid: Thread ID (PID)
 *  rbar: MPU RBAR
 *  rasr: MPU RASR
 *  useptr: The "usable" data pointer returned by the mpu allocator, used for
 *    address lookup
 *  freeptr: The pointer returned by the mpu allocator for freeing the memory
 */
int PagerAddEntry (unsigned int tid, unsigned int rbar, unsigned int rasr,
	unsigned int useptr, unsigned int freeptr);

/*
 * Free the first entry found matching the tid and address
 */
int PagerFreeEntry (unsigned int tid, unsigned int useptr);

/*
 * Page walk function
 *   address: The address that generated the fault
 *   trc: The thread control structure of the process
 *   out: Stack space to store the to-be-set MPU registers
 *   dfsr: Data Fault Status Register used to check access permissions
 */
unsigned int *PagerWalk (unsigned int address, struct ThreadControl *trc,
	unsigned int *out, unsigned int dfsr);

#endif
