/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by erlendjs on 06/05/16.
//

#ifndef NETWORK_LAYER2_ARP_H
#define NETWORK_LAYER2_ARP_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "net/enet/layer2/dataframe.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
/*
 * Initialize the ARP component.
 * Clears the cache table if enabled and stores our address.
 */
void ARPInit (unsigned short myprotocoladdress);

/*
 * ARP Resolve.
 * Tries to find the hardware address for the specified protocol and protocol
 * address. Return value zero means that the request failed, otherwise it is
 * valid. If the cache table is enabled, it is searched first and no outside
 * request is done if an address is found in the table.
 *
 * Warning: Blocking call!
 */
unsigned short ARPResolve (unsigned short remoteprotocoladdress);

/*
 * Process incoming ARP packets.
 * It answers other nodes that want our hw address and gathers replies to our
 * own requests.
 *
 * Note: Takes ownership
 */
void ARPProcess (DataFrame *frame);

#endif
