/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef LAYER2_INTERFACE_H
#define LAYER2_INTERFACE_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "../config/live/network-config.h"
#include "net/enet/layer2/dataframe.h"

#if NETWORK_DEBUG_INTERFACE
#include NETWORK_DEBUG_INC
#endif

#ifdef NETWORK_PLATFORM_INC
#include NETWORK_PLATFORM_INC
#endif

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Statistic recording
#define LAYER2_DOSTATS 1

///////////////////////////////////////////////////////////////////////////////
// Debug definitions
///////////////////////////////////////////////////////////////////////////////
#if NETWORK_DEBUG_INTERFACE
	#define IFDEBUG_OTHER(str) str
#else
	#define IFDEBUG_OTHER(str)
#endif

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#if LAYER2_DOSTATS
	#define IFSTATS(arg) arg
#else
	#define IFSTATS(arg)
#endif

///////////////////////////////////////////////////////////////////////////////
// Physical layer driver structures
///////////////////////////////////////////////////////////////////////////////

// Statistics
typedef struct Layer2Statistics
{
	unsigned int framesreceived;
	unsigned int framessent;

	unsigned int bytesreceived;
	unsigned int bytessent;

	unsigned int error_rxfifofull;
	unsigned int error_pumpallocerr;
	unsigned int error_sendframedma;
	unsigned int error_txfifofull;
	unsigned int error_framesize;
	unsigned int error_checksum;
}Layer2Statistics;

// Layer 2 driver type
typedef struct Layer2
{
	// Next inferface
	struct Layer2 *next;

	// Precomputed interface ID
	struct Layer2 *interfaceid;

	// Link state
	unsigned char linkstate;

	// Local receive circular buffer
	unsigned char rxbuffer[RS485_RXBUFF_SIZE];
	unsigned int writepos;
	unsigned int readfrompos;
	unsigned int readtopos;

	// Local transmit FIFO
	DataFrame *txframes[RS485_TXFIFO_SIZE];
	unsigned char txframesread;
	unsigned char txframeswrite;

	// Statistics, included if enabled
	IFSTATS (Layer2Statistics stats);
}Layer2;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void AddInterface (Layer2 *interface);

///////////////////////////////////////////////////////////////////////////////
// Interface
///////////////////////////////////////////////////////////////////////////////

/*
 * Initializer
 * Does UART setup, DMA setup and starts reception.
 */
void Layer2Init (Layer2 *l);

/*
 * Processes incoming data and starts outgoing dma transfers.
 */
void Layer2Pump (Layer2 *l);

/*
 * User implemented
 */
int Layer2SendFrameInternal (Layer2 *l, DataFrame *frame);

/*
 * Set speed
 */
void Layer2SetSpeed (Layer2 *l, unsigned int speed);

/*
 * Interface OK?
 */
int Layer2IsUp (Layer2 *l);

/*
 * Link up?
 */
int Layer2LinkUp (Layer2 *l);

#endif
