/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by erlendjs on 27/04/16.
//

#ifndef NETWORK_LAYER2_LINKSPEED_H
#define NETWORK_LAYER2_LINKSPEED_H

/*
 * States:
 *  0 - Sequence completed, monitoring link->IsUp
 *  1 - Ready to start sequence, still monitoring link->IsUp
 *  2 - Sent discovery, listening for list reply or timeout event
 *  3 - Got list and sent set speed, waiting for Update tick to send data
 *  4 - Sent test packet, waiting for reply
 *
 *  10 - Test packet failed. Waiting for timeout.
 *
 *  254 - Got set speed, speed has been set and is waiting for testdata
 *  255 - Sent speed list, waiting for set speed
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "../config/live/network-config.h"
#include "net/enet/layer2/dataframe.h"
#include "net/enet/layer2/interface.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// This MCU's address
#define LAYER2LINKSPEED_ADDRESS LAYER2_ADDRESS

// Link address
#define LAYER2LINKSPEED_DSTADDRESS 0x3030

// Protocol numer
#define LAYER2LINKSPEED_PROTOCOL '3'

// Packet types
#define LAYER2LINKSPEED_PTYPE_DISCOVERY 'd'
#define LAYER2LINKSPEED_PTYPE_SPEEDLIST 'l'
#define LAYER2LINKSPEED_PTYPE_SETSPEED 's'
#define LAYER2LINKSPEED_PTYPE_TEST 't'
#define LAYER2LINKSPEED_PTYPE_TESTREPLY 'r'
#define LAYER2LINKSPEED_PTYPE_SPEEDCONFIRM 'c'

// Speedlist
#define LAYER2LINKSPEED_SPEEDLIST_DEFAULT 254

// Timeouts
#define LAYER2LINKSPEED_RESTARTTIMEOUT 3000		// Used by discovery
#define LAYER2LINKSPEED_SENDTESTFRAMEWAIT 100	// Between receiving list and first test frame
#define LAYER2LINKSPEED_RECVTESTTIMEOUT 1000	// Timeout from sending test to receiving it
#define LAYER2LINKSPEED_REPLIEDTIMEOUT 3000		// Timeout from replying to test
#define LAYER2LINKSPEED_SETSPEEDTIMEOUT 1000	// Timeout from receiving set speed
#define LAYER2LINKSPEED_YIELDTIMEOUT 5000		// When receiving from higher ranked, stay put

// Sequence
#define LAYER2LINKSPEED_SEQUENCESTART 10

///////////////////////////////////////////////////////////////////////////////
// Protocol structures
///////////////////////////////////////////////////////////////////////////////

// Data rate enumeration
enum DataRate
{
	UARTSPEED_DEFAULT = 19200,
	UARTSPEED_19200 = 19200,
	UARTSPEED_57600 = 57600,
	UARTSPEED_115200 = 115200,
	UARTSPEED_250000 = 250000,
	UARTSPEED_500000 = 500000,
	UARTSPEED_1000000 = 1000000,
	UARTSPEED_1500000 = 1500000,
	UARTSPEED_2000000 = 2000000
};

///////////////////////////////////////////////////////////////////////////////
// Structure definitions
///////////////////////////////////////////////////////////////////////////////
typedef struct Layer2Linkspeed
{
	Layer2 *link;
	unsigned char state;
	unsigned char sequence;
	unsigned short servingremote;
	unsigned int time;
	unsigned int settingspeed;
	unsigned int speeds[8];
}Layer2Linkspeed;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void Layer2LinkspeedConstructor (Layer2Linkspeed *l, Layer2 *interface);
void Layer2LinkspeedUpdate (Layer2Linkspeed *l);
void Layer2LinkspeedProcess (Layer2Linkspeed *l, DataFrame *frame); // Note: takes ownership
int Layer2LinkspeedLinkIsReady (Layer2Linkspeed *l);

#endif
