/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by erlendjs on 10/05/16.
//

#ifndef NETWORK_LAYER4_TCPUDP_H
#define NETWORK_LAYER4_TCPUDP_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "net/enet/layer2/dataframe.h"
#include "posix/arpa/inet.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define NET_IP_BROADCAST 0xFAFA

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////

// This structure defines the data members of a DataFrame when used in UDP
typedef struct UDPCast
{
	unsigned char reserved[6]; 		// layer2 packet header
	unsigned short destination;		// layer3 ip
	unsigned short source;			// layer3 ip
	unsigned char ttl;				// layer3 ip
	unsigned char payloadprotocol;	// layer3 ip
	unsigned char dstport;			// layer4 udp
	unsigned char srcport;			// layer4 udp
	unsigned char payload[242];		// layer4 udp
	unsigned int reserved2;
}UDPCast;

// UDP Socket definition
typedef struct UDPSocket
{
	unsigned short ip;			// Destination IP
	unsigned char dstport;		// Destination port
	unsigned char srcport;		// Source port
	unsigned short hwaddr;		// ARP Cache
	struct UDPSocket *next;		// For internal linked list of sockets
	DataFrame *frames;			// Linked list of frames pending reception
	DataFrame *tail;			// Tail of the linked list
}UDPSocket;

///////////////////////////////////////////////////////////////////////////////
// Functions: UDP Socket member definitions
///////////////////////////////////////////////////////////////////////////////
/*
 * Initializes a socket with the supplied data and adds it to some internal
 * structures, making it usable.
 */
int UDPInit (UDPSocket *s, unsigned short ipaddress, unsigned char dst, unsigned char src);

/*
 * Frees data allocated within the socket. Does not free the socket itself.
 */
void UDPRemove (UDPSocket *s);

int UDPFindFreePort (void);
int UDPSetPort (int port);
int UDPClearPort (int port);

/*
 * Send data of len bytes using socket that, optionally sending it back to
 * receiver defined by reference.
 */
int UDPSend (UDPSocket *s, const unsigned char *data, int len, const struct sockaddr *addr);

int UDPRecv (UDPSocket *s, unsigned char *data, int len, struct sockaddr *addr);

///////////////////////////////////////////////////////////////////////////////
// Functions: General UDP API function definitions
///////////////////////////////////////////////////////////////////////////////
/*
 * Process IP frame.
 */
void IPProcess (DataFrame *f);

#endif
