/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef PANIC_H
#define PANIC_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "sched/threadcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

/*
 * These definitions identifies if the fault occured in the kernel or in
 * userspace.
 */
#define PANIC_STACKTYPE_KERNEL 0
#define PANIC_STACKTYPE_USER 1

/*
 * These definitions identify the fault source. They are mostly relevant
 * for platform specific error handling.
 */
#define PANIC_FAULTTYPE_KERNEL 0

// Cortex-M
#define PANIC_FAULTTYPE_HARDFAULT 1
#define PANIC_FAULTTYPE_MEMMANAGE 2
#define PANIC_FAULTTYPE_BUSFAULT 3
#define PANIC_FAULTTYPE_USAGEFAULT 4

// Cortex-R
#define PANIC_FAULTTYPE_UNDEFINST 1
#define PANIC_FAULTTYPE_PREFABORT 2
#define PANIC_FAULTTYPE_DATAABORT 3

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Print the specified message and stall the system for easy debug access.
 *
 * WARNING:
 *  Calling this is essentially declaring the system as dead. You cannot return
 *  from this, except for hardware resets.
 */
void Panic (const char *message);

/*
 * The more generic handler for different panic types. Not to be used directly.
 * Should be defined in the platform code.
 */
void PanicInternal (unsigned int *stack, int stacktype, int faulttype);

/*
 * Print crash report of a userspace thread
 */
void PanicPrintReport (struct ThreadControl *ctc);

#endif
