/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef PLATFORM_H
#define PLATFORM_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "sched/threadcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Exports
///////////////////////////////////////////////////////////////////////////////
extern volatile unsigned int systemTick;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 *
 */
void PlatformSetInterruptPriority (int irqnum, int priority);

/*
 *
 */
void PlatformEnableInterrupt (int irqnum);

/*
 * Initialize the system hardware on startup
 */
void PlatformInitialize (void);

/*
 * Configure and start the system tick, with a rate of 1000 Hz.
 * Must be called whenever the clock rate changes to maintain the
 * correct rate.
 */
void PlatformInitTick (void);

/*
 * Get the current tick counter value
 */
inline unsigned int PlatformGetTick (void)
{
	return systemTick;
}

/*
 *
 */
void PlatformInitMemory (void);

/*
 * Store platform specific crash registers
 */
void PlatformStoreCrashRegs (struct ThreadControl *ctc);

#endif
