/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FCNTL_H
#define FCNTL_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "posix/sys/types.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// fcntl commands
// Defined in sys/ioctl.h

// fcntl flags: file descriptor
#define FD_CLOEXEC		(0 << 0)	// Not supported

// fcntl flags: record locking
#define F_RDLCK			1			// Not supported
#define F_UNLCK			2			// Not supported
#define F_WRLCK			3			// Not supported

// open flags: file creation
#define O_CREAT         (1 << 9)
#define O_EXCL			(0 << 0)	// Not supported
#define O_NOCTTY		(0 << 0)	// Not supported
#define O_TRUNC         (1 << 10)

// open and fcntl flags: file status
#define O_APPEND        (1 << 3)
#define O_DSYNC			(0 << 0)	// Not supported
#define O_NONBLOCK		(1 << 4)	// Not supported
#define O_RSYNC			(0 << 0)	// Not supported
#define O_SYNC			(0 << 0)	// Not supported

// mask for use with file access modes
#define O_ACCMODE		(0 << 0)	// Not supported

// open file access modes
#define O_RDONLY        (0 << 0)
#define O_RDWR          (1 << 1)
#define O_WRONLY        (1 << 0)

// ??
#define FASYNC			0x1000

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

int open (const char*, int, ...);

#endif
