/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef IN_H
#define IN_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "posix/sys/types.h"
#include "posix/sys/socket.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
typedef unsigned short in_port_t;
typedef unsigned int in_addr_t;

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
struct in_addr
{
	in_addr_t s_addr;
};

struct sockaddr_in
{
	sa_family_t sin_family;
	in_port_t sin_port;
	struct in_addr sin_addr;
};

struct in6_addr
{
	unsigned char s6_addr[16];
};

struct sockaddr_in6
{
	sa_family_t sin6_family;
	in_port_t sin6_port;
	unsigned int sin6_flowinfo;
	struct in6_addr sin6_addr;
	unsigned int sin6_scope_id;
};

struct ipv6_mreq
{
	struct in6_addr ipv6mr_multiaddr;
	unsigned int opv6mr_interface;
};

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "posix/arpa/inet.h"

///////////////////////////////////////////////////////////////////////////////
// Externs
///////////////////////////////////////////////////////////////////////////////
extern const struct in6_addr in6addr_any;
extern const struct in6_addr in6addr_loopback;

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Constants (TODO)
#define IN6ADDR_ANY_INIT
#define IN6ADDR_LOOPBACK_INIT

// level argument
#define IPPROTO_IP		1
#define IPPROTO_IPV6	2
#define IPPROTO_ICMP	3
#define IPPROTO_RAW		4
#define IPPROTO_TCP		5
#define IPPROTO_UDP		6

// For passing to bind
#define INADDR_ANY		0

// For destination address passed to connect, sendmsg, sendto
#define INADDR_BROADCAST	1

// Length of ipv4 string
#define INET_ADDRSTRLEN	16

// Length of ipv6 string
#define INET6_ADDRSTRLEN 46

// option_name argument
#define IPV6_JOIN_GROUP
#define IPV6_LEAVE_GROUP
#define IPV6_MULTICAST_HOPS
#define IPV6_MULTICAST_IF
#define IPV6_MULTICAST_LOOP
#define IPV6_UNICAST_HOPS
#define IPV6_V6ONLY

// Macros for testing for special ipv6 addresses
#define IN6_IS_ADDR_UNSPECIFIED
#define IN6_IS_ADDR_LOOPBACK
#define IN6_IS_ADDR_MULTICAST
#define IN6_IS_ADDR_LINKLOCAL
#define IN6_IS_ADDR_SITELOCAL
#define IN6_IS_ADDR_V4MAPPED
#define IN6_IS_ADDR_V4COMPAT
#define IN6_IS_ADDR_MC_NODELOCAL
#define IN6_IS_ADDR_MC_LINKLOCAL
#define IN6_IS_ADDR_MC_SITELOCAL
#define IN6_IS_ADDR_MC_ORGLOCAL
#define IN6_IS_ADDR_MC_GLOBAL

#endif
