/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SPAWN_H
#define SPAWN_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "posix/sys/types.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define POSIX_SPAWN_SETPGROUP 1
#define POSIX_SPAWN_NFACTIONS 2

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct posix_spawnattr_t
{
	int flags;
	int pgroup;
}posix_spawnattr_t;

typedef struct file_action
{
	int fildes;
	char *path;
	int oflag;
	int mode;
	char inuse;	// Is this entry in use or not?
}file_action;

typedef struct posix_spawn_file_actions_t
{
	int nfileactions;
	file_action fa[POSIX_SPAWN_NFACTIONS];
}posix_spawn_file_actions_t;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int posix_spawn (pid_t*, const char*, const posix_spawn_file_actions_t*,
	const posix_spawnattr_t*, char **, char **);

#endif
