/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SOCKET_H
#define SOCKET_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "posix/sys/types.h"
//#include "sys/uio.h"		// for iovec later on

///////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////////////////////////////////////////////
typedef unsigned int socklen_t;
typedef unsigned int sa_family_t;

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
struct sockaddr
{
	sa_family_t sa_family;
	char sa_data[12];
};

struct sockaddr_storage
{
	sa_family_t sa_family;
	char sa_data[12+16];
};

struct msghdr
{
	void *msg_name;
	socklen_t msg_namelen;
	struct iovec *msg_iov;
	int msg_iovlen;
	void *msg_control;
	socklen_t msg_controllen;
	int msg_flags;
};

struct cmsghdr
{
	socklen_t cmsg_len;
	int cmsg_level;
	int cmsg_type;
};

struct linger
{
	int l_onoff;
	int l_linger;
};

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// Bleh
#define SCM_RIGHTS 0
#define CMSG_DATA(cmsg) 0
#define CMSG_NXTHDR(cmsg) 0
#define CMSG_FIRSTHDR(mhdr) 0

// Socket types
#define SOCK_DGRAM		0	// Datagram socket
#define SOCK_RAW		1	// Raw protocol ingerface
#define SOCK_SEQPACKET	2	// Sequenced-packet socket
#define SOCK_STREAM		3	// Byte-stream socket

// Level argument for setsockopt and getsockopt
#define SOL_SOCKET		1

// option_name argment for getsockopt and setsockopt
#define SO_ACCEPTCONN	1
#define SO_BROADCAST	2
#define SO_DEBUG		3
#define SO_DONTROUTE	4
#define SO_ERROR		5
#define SO_KEEPALIVE	6
#define SO_LINGER		7
#define SO_OOBINLINE	8
#define SO_RCVBUF		9
#define SO_RCVLOWAT		10
#define SO_RCVTIMEO		11
#define SO_REUSEADDR	12
#define SO_SNDBUF		13
#define SO_SNDLOWAT		14
#define SO_SNDTIMEO		15
#define SO_TYPE			16

// Maximum backlog queue length
#define SOMAXCONN		1

// msg_flags in msghdr or flags in function calls
#define MSG_CTRUNC		1
#define MSG_DONTROUTE	2
#define MSG_EOR			3
#define MSG_OOB			4
#define MSG_NOSIGNAL	5
#define MSG_PEEK		6
#define MSG_TRUNC		7
#define MSG_WAITALL		8

// Net types
#define AF_INET			4
#define AF_INET6		6
#define AF_UNIX			1
#define AF_UNSPEC		0

// Other
#define SHUT_RD			1
#define SHUT_RDWR		3
#define SHUT_WR			2

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int accept (int, struct sockaddr *restrict, socklen_t *restrict);
int bind (int, const struct sockaddr *, socklen_t);
int connect (int, const struct sockaddr *, socklen_t);
int getpeername (int, struct sockaddr *restrict, socklen_t *restrict);
int getsockname (int, struct sockaddr *restrict, socklen_t *restrict);
int getsockopt (int, int, int, void *restrict, socklen_t *restrict);
int listen (int, int);
ssize_t recv (int, void *, size_t, int);
ssize_t recvfrom (int, void *restrict, size_t, int, struct sockaddr *restrict, socklen_t *restrict);
ssize_t recvmsg (int, struct msghdr *, int);
ssize_t send (int, const void *, size_t, int);
ssize_t sendmsg (int, const struct msghdr *, int);
ssize_t sendto (int, const void *, size_t, int, const struct sockaddr *, socklen_t);
int setsockopt (int, int, int, const void*, socklen_t);
int shutdown (int, int);
int sockatmark (int);
int socket (int, int, int);
int socketpair (int, int, int, int[2]);

#endif
