/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "process/processcontrol.h"
#include "sched/threadcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define MAX_PROCESSDATA 8

///////////////////////////////////////////////////////////////////////////////
// Exports
///////////////////////////////////////////////////////////////////////////////
extern struct ProcessControl processdata[MAX_PROCESSDATA];

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Allocate space for one process control structure. This may be from a
 * static array. If the allocation fails, the return value is null. Note that
 * the new structure is marked as in use, but no PID is allocated for it.
 *
 * This function is not thread safe.
 */
struct ProcessControl *ProcessAllocate (void);

/*
 * Free a process allocated with the function above.
 */
void ProcessFree (struct ProcessControl *prc);

/*
 * Find a process by its PID. The return value is zero if not found.
 */
struct ProcessControl *ProcessFindByPid (int pid);
