/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "sched/threadcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*
 * Allocate a new process control block and initialize it with the supplied
 * function and stack. The process is given a PID, has the root directory
 * as its working directory and is set to privileged access. The process is
 * then started with normal priority.
 *
 * Returns a control block if successful, null otherwise.
 */
struct ProcessControl *ProcessCreateInternal (void(*func)(void),
	unsigned int *stack,
	int stacksize);

/*
 * Close all open file descriptors, including stdin and stdout
 */
void ProcessFileCleanup (struct ProcessControl *p);

/*
 * Notify owner on status change
 */
void ProcessNotifyOnStatusChange (struct ThreadControl *ctc, int terminated);

/*
 * Send a signal to some process
 */
int ProcessSendSignal (int pid, int sig);

/*
 * Handle a signal that was set as ignored
 */
int ProcessHandleIgnoredSignal (struct ProcessControl *p, int sig);

/*
 * Cleanup after crashed process
 */
int ProcessCrashRecover (struct ThreadControl *ctc, unsigned int args);
