/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef THREADCONTROL_H
#define THREADCONTROL_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "sched/list.h"
#include "../../config/live/archconfig.h"

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
struct ThreadControl
{
	// Pointer to current top of stack, must be first
	unsigned int *stacktop;

	// Privileged or not, set to 1 for unprivileged, must be second
	unsigned int privileged;

	// MPU data, up to 4 regions, must be third
	unsigned int mpuregs[MPU_REGS_NEEDED];

	// List entry for scheduler lists
	// Note that the parent pointer points back here, not to parent process
	struct ListItem listentry;

	// Pointer to bottom of stack, used to check for stack overflow in kernel mode
	unsigned int *stackbottom;

	// Program entry point and callback for deferred operations
	union {
		void (*entrypoint)(void);
		int (*callback)(struct ThreadControl*, unsigned int);
	};

	// Parameters initially used for startup. Later reused by signals.
	// In case of a crash, they are reused for hfsr, cfsr and bfar.
	unsigned int param1;
	unsigned int param2;
	unsigned int param3;

	// Priority (see definitions above)
	// In case of a crash, priority is reused and set to 4
	unsigned char priority;

	// Current state of the process, see SCHED_STATUS definitions above
	unsigned char state;
	unsigned char statechanged;

	// Used by SchedSelectNext to either enter (2) or leave (1) the signal handler
	unsigned char sigstate;

	// Signal mask, for enabling signals. 0 is the default ignore action.
	// In case of a crash, sigmask is reused as mmar
	unsigned int sigmask;

	// Signal pending bits
	// In case of a crash, sigpending is reused for faulttype
	unsigned int sigpending;

	// Signal handler and signal trampoline function pointers
	void (*sigh)(int);
	void (*sigt)(int);

	// Copy of stacktop used when leaving a signal handler
	unsigned int *stackstored;

	// Pointer to the ProcessControl structure, containing variables used by
	// the rest of the system but not the scheduler. Keeping it separate allows
	// for lightweight use of the scheduler, possibly threads later on.
	struct ProcessControl *pctrl;

	// TODO: Priority inversion, time counter
};

#endif
