/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * This is where the CPU starts executing when it encounters an SVC
 * instruction. It essentially just runs the correct function in the dispatch
 * table.
 */

.section .text
.syntax unified
.cpu cortex-m4
.thumb

.weak FaultHandler
.type FaultHandler, %function
FaultHandler:
	tst lr, #4				@ Test bit 2 of lr, true when we came from PSP
	bne loadfrompsp
	mrs r0, msp				@ *stack = MSP
	mov r1, #0				@ stacktype = 0
	b PanicInternal
loadfrompsp:
	mrs r0, psp				@ *stack = PSP
	mov r1, #1				@ stacktype  = PANIC_STACKTYPE_USER
	stmdb sp!, {lr}			@ Stack return register
	bl SchedCrashRecover	@ Attempt crash recovery
	ldmia sp!, {lr}			@ Unstack return reg
	b SchedContextNext		@ Jump into scheduler and restore a different process
.size FaultHandler, .-FaultHandler

.weak HardFaultHandler
.type HardFaultHandler, %function
HardFaultHandler:
	mov r2, #1
	b FaultHandler
.size HardFaultHandler, .-HardFaultHandler

.weak MemManageHandler
.type MemManageHandler, %function
MemManageHandler:
	mov r2, #2
	b FaultHandler
.size MemManageHandler, .-MemManageHandler

.weak BusFaultHandler
.type BusFaultHandler, %function
BusFaultHandler:
	mov r2, #3
	b FaultHandler
.size BusFaultHandler, .-BusFaultHandler

.weak UsageFaultHandler
.type UsageFaultHandler, %function
UsageFaultHandler:
	mov r2, #4
	b FaultHandler
.size UsageFaultHandler, .-UsageFaultHandler
