/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Include
///////////////////////////////////////////////////////////////////////////////
#include "sched/threadcontrol.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void InitializeStack (struct ThreadControl *t)
{
	unsigned int *top = t->stacktop;

	top--; *top = 0x01000000; // xPSR: Set the thumb bit in the xPSR
	top--; *top = (unsigned int) t->entrypoint; // PC: Set to entry point
	top--; *top = 0x00000000; // LR: Return address
	top--; *top = 0; // R12
	top--; *top = 0; // R3
	top--; *top = t->param3; // R2: Normally environ
	top--; *top = t->param2; // R1: Normally argv
	top--; *top = t->param1; // R0: Normally argc
	top--; *top = 0xFFFFFFFD; // Return value used: To thread mode, no float, state and exec from PSP
	top--; *top = 0; // R11
	top--; *top = 0; // R10
	top--; *top = 0; // R9
	top--; *top = 0; // R8
	top--; *top = 0; // R7
	top--; *top = 0; // R6
	top--; *top = 0; // R5
	top--; *top = 0; // R4

	t->stacktop = top;
}

void SchedSignalStackSetup (struct ThreadControl *thc)
{
	unsigned int *top = thc->stacktop;

	thc->stackstored = top;

	top--; *top = 0x01000000; // xPSR: Set the thumb bit in the xPSR
	top--; *top = (unsigned int) thc->sigh; // PC: Set to entry point
	top--; *top = (unsigned int) thc->sigt; // LR: Return address
	top--; *top = 0; // R12
	top--; *top = 0; // R3
	top--; *top = 0; // R2
	top--; *top = 0; // R1
	top--; *top = thc->param1; // R0
	top--; *top = 0xFFFFFFFD; // Return value used: To thread mode, no float, state and exec from PSP
	top--; *top = 0; // R11
	top--; *top = 0; // R10
	top--; *top = 0; // R9
	top--; *top = 0; // R8
	top--; *top = 0; // R7
	top--; *top = 0; // R6
	top--; *top = 0; // R5
	top--; *top = 0; // R4

	thc->stacktop = top;
	thc->sigstate = 0;

	// Clear the signal pending bit
	thc->sigpending &= ~(1 << (thc->param1 - 1));
}

void SchedYield (void)
{
#define NVIC_INTCTRL (*((volatile unsigned int*) 0xE000ED04))
	NVIC_INTCTRL = 1 << 28;
	__asm volatile("dsb");
	__asm volatile("isb");
}
