/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * This is where the CPU starts executing when it encounters an SVC
 * instruction. It essentially just runs the correct function in the dispatch
 * table.
 */

.section .text
.syntax unified
.cpu cortex-m4
.thumb

.weak SVCHandler
.type SVCHandler, %function
SVCHandler:
	tst lr, #4					@ Load the correct stack pointer to r1
	ite eq
	mrseq r1, msp
	mrsne r1, psp

	ldr r2, [r1, #24]			@ Add 6 words (6*4 = 24) to point to the stacked pc
	ldrb r2, [r2, #-2]			@ Get the stacked pc
								@ Point to the svc immediate, endianness makes it 2
								@ Load the svc argument

	and r2, r2, #0x3F			@ Make sure to keep index within jump table bounds
	ldr r3, =svcJumpTable		@ Load address of jump table
	ldr r3, [r3, r2, lsl #2]	@ Load entry by shifting argument two times

	ldr r0, =currentthc			@ Load current thread
	ldr r0, [r0]

	push {r1, lr}
	blx r3						@ Branch and link to entry
	pop {r1, lr}
	str r0, [r1]
	bx lr
.size SVCHandler, .-SVCHandler
