/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

.syntax unified
.cpu cortex-r4
.thumb

.section .text.KernelStackInit
.global KernelStackInit
.type KernelStackInit, %function
KernelStackInit:
	@ Cortex-R4 and Cortex-R4F Technical Reference Manual, 2.7.11
	ldr r0, =kernelstack

	@ Fast interrupt (not used, set same as regular)
	cps #17
	mov sp, r0			@0x08001000

	@ Regular interrupt
	cps #18
	mov sp, r0			@0x08001000

	@ Supervisor
	cps #19
	sub r0, r0, #0x100
	mov sp, r0			@0x08000f00

	@ Abort
	cps #23
	sub r0, r0, #0x400
	mov sp, r0			@0x08000b00

	@ Undefined
	cps #27
	sub r0, r0, #0x100
	mov sp, r0			@0x08000a00

	@ System (same stack as user)
	cps #31
	sub r0, r0, #0x100
	mov sp, r0			@0x08000900

	ldr r0, =_ramstart
	ldr r1, =_ramend
	mov r2, #0

KernelResetEntryZero:
	str r2, [r0], #4
	cmp r0, r1
	bcc KernelResetEntryZero

	ldr r0, =kernelstackstart
	ldr r1, =kernelstack
	ldr r2, =0xDEADBEEF

KernelResetEntryStack:
	str r2, [r0], #4
	cmp r0, r1
	bcc KernelResetEntryStack

	ldr r0, =_dataloadfrom
	ldr r1, =_datastart
	ldr r2, =_dataend

KernelResetEntryCopy:
	ldr r3, [r0], #4
	str r3, [r1], #4
	cmp r1, r2
	bcc KernelResetEntryCopy

	bl KernelInit

KernelResetEntryError:
	b KernelResetEntryError
	nop
.size KernelStackInit, .-KernelStackInit
