/*--------------------------------------------------------------------------
 sys_core.s

 Copyright (C) 2009-2016 Texas Instruments Incorporated - www.ti.com


  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

    Neither the name of Texas Instruments Incorporated nor the names of
    its contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-------------------------------------------------------------------------*/

.section .text
.syntax unified
.cpu cortex-r4
.arm

.weak CoreInitRegisters
.type CoreInitRegisters, %function
CoreInitRegisters:
	@ After reset, the CPU is in the Supervisor mode (M = 10011)
	mov r0, lr
	mov r1, #0x0000
	mov r2, #0x0000
	mov r3, #0x0000
	mov r4, #0x0000
	mov r5, #0x0000
	mov r6, #0x0000
	mov r7, #0x0000
	mov r8, #0x0000
	mov r9, #0x0000
	mov r10, #0x0000
	mov r11, #0x0000
	mov r12, #0x0000
	mov r13, #0x0000
	mrs r1, cpsr
	msr spsr_cxsf, r1
	@ Switch to FIQ mode (M = 10001)
	cps #17
	mov lr, r0
	mov r8, #0x0000
	mov r9, #0x0000
	mov r10, #0x0000
	mov r11, #0x0000
	mov r12, #0x0000
	mrs r1, cpsr
	msr spsr_cxsf, r1
	@ Switch to IRQ mode (M = 10010)
	cps #18
	mov lr, r0
	mrs r1,cpsr
	msr spsr_cxsf, r1             @ Switch to Abort mode (M = 10111)
	cps #23
	mov lr, r0
	mrs r1,cpsr
	msr spsr_cxsf, r1             @ Switch to Undefined Instruction Mode (M = 11011)
	cps #27
	mov lr, r0
	mrs r1,cpsr
	msr spsr_cxsf, r1             @ Switch to System Mode ( Shares User Mode registers ) (M = 11111)
	cps #31
	mov lr, r0
	mrs r1,cpsr
	msr spsr_cxsf, r1

	mrc   p15,     #0x00,      r2,       c1, c0, #0x02
	orr   r2,      r2,         #0xF00000
	mcr   p15,     #0x00,      r2,       c1, c0, #0x02
	mov   r2,      #0x40000000
	fmxr  fpexc,   r2

	fmdrr d0, r1, r1
	fmdrr d1, r1, r1
	fmdrr d2, r1, r1
	fmdrr d3, r1, r1
	fmdrr d4, r1, r1
	fmdrr d5, r1, r1
	fmdrr d6, r1, r1
	fmdrr d7, r1, r1
	fmdrr d8, r1, r1
	fmdrr d9, r1, r1
	fmdrr d10, r1, r1
	fmdrr d11, r1, r1
	fmdrr d12, r1, r1
	fmdrr d13, r1, r1
	fmdrr d14, r1, r1
	fmdrr d15, r1, r1

	bl next1
next1:
	bl next2
next2:
	bl next3
next3:
	bl next4
next4:
	bx r0


/*-------------------------------------------------------------------------------*/
@ Work Around for Errata CORTEX-R4#57:
@
@ Errata Description:
@            Conditional VMRS APSR_Nzcv, FPSCR May Evaluate With Incorrect Flags
@ Workaround:
@            Disable out-of-order single-precision floating point
@            multiply-accumulate instruction completion

.weak _errata_CORTEXR4_57_
.type _errata_CORTEXR4_57_, %function
_errata_CORTEXR4_57_:
	push {r0}
	mrc p15, #0, r0, c15, c0, #0 @ Read Secondary Auxiliary Control Register
	orr r0, r0, #0x10000         @ Set BIT 16 (Set DOOFMACS)
	mcr p15, #0, r0, c15, c0, #0 @ Write Secondary Auxiliary Control Register
	pop {r0}
	bx lr

/*-------------------------------------------------------------------------------*/
@ Work Around for Errata CORTEX-R4#66:
@
@ Errata Description:
@            Register Corruption During A Load-Multiple Instruction At
@            an Exception Vector
@ Workaround:
@            Disable out-of-order completion for divide instructions in
@            Auxiliary Control register

.weak _errata_CORTEXR4_66_
.type _errata_CORTEXR4_66_, %function
_errata_CORTEXR4_66_:
	push {r0}
	mrc p15, #0, r0, c1, c0, #1 @ Read Auxiliary Control register
	orr r0, r0, #0x80           @ Set BIT 7 (Disable out-of-order completion
		                        @ for divide instructions.)
	mcr p15, #0, r0, c1, c0, #1 @ Write Auxiliary Control register
	pop {r0}
	bx lr
