/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

.syntax unified
.cpu cortex-r4
.fpu softvfp
.arm

.section .text.SchedStartAsm
.global SchedStartAsm
.type SchedStartAsm, %function
SchedStartAsm:
	cpsie i
	cpsie f
	svc 63
	nop
.size SchedStartAsm, .-SchedStartAsm

.section .text.SchedStartSvc
.global SchedStartSvc
.type SchedStartSvc, %function
SchedStartSvc:
	mov r3, #0					@ Base flash region, for privileged access
	mov r4, #0
	mov r5, #0x29
	ldr r6, =0x506
	mcr p15, 0, r3, c6, c2, 0	@ Set region
	mcr p15, 0, r4, c6, c1, 0	@ Set address
	mcr p15, 0, r6, c6, c1, 4	@ Set access rights
	mcr p15, 0, r5, c6, c1, 2	@ Set size and enable

	mov r3, #1					@ Base ram region, for privileged access
	mov r4, #0x08000000
	mov r5, #0x23
	ldr r6, =0x106
	mcr p15, 0, r3, c6, c2, 0	@ Set region
	mcr p15, 0, r4, c6, c1, 0	@ Set address
	mcr p15, 0, r6, c6, c1, 4	@ Set access rights
	mcr p15, 0, r5, c6, c1, 2	@ Set size and enable

	mov r3, #2					@ Base peripheral region, for privileged access
	mov r4, #0xF0000000
	mov r5, #0x37
	ldr r6, =0x110
	mcr p15, 0, r3, c6, c2, 0	@ Set region
	mcr p15, 0, r4, c6, c1, 0	@ Set address
	mcr p15, 0, r6, c6, c1, 4	@ Set access rights
	mcr p15, 0, r5, c6, c1, 2	@ Set size and enable

	mrc p15, #0, r0, c1, c0, #0	@ Enable MPU
	orr r0, r0, #1
	dsb
	mcr p15, #0, r0, c1, c0, #0
	isb

							@ TODO Start systick irq here
	ldr r0, =kernelstack	@ Load top of kernel stack
	sub r0, r0, #0x100		@ Reset SVC handler stack
	mov sp, r0

	ldr	r3, =currentthc		@ Get currentthc pointer
	ldr r1, [r3]
	ldr r0, [r1]			@ Get stacktop

	ldmia r0!, {r1,lr}		@ Unstack CPSR and PC
	msr spsr, r1			@ Put CPSR back in SVC SPSR
	cps #31					@ Switch to system mode
	ldmia r0, {r0-lr}		@ Unstack R0-LR
	cps #19					@ Switch back to svc mode
	subs pc, lr, #4			@ Return from svc back to thread
.size SchedStartSvc, .-SchedStartSvc

.section .text.SchedEnterCritical
.global SchedEnterCritical
.type SchedEnterCritical, %function
SchedEnterCritical:
	mrs r0, cpsr
	cpsid i
	bx lr
.size SchedEnterCritical, .-SchedEnterCritical

.section .text.SchedExitCritical
.global SchedExitCritical
.type SchedExitCritical, %function
SchedExitCritical:
	tst r0, #0x80
	bne abortexitecritical
	cpsie i
abortexitecritical:
	bx lr
.size SchedExitCritical, .-SchedExitCritical
