/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Include
///////////////////////////////////////////////////////////////////////////////
#include "sched/threadcontrol.h"
#include "arch/tms570ls1224/platform.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void InitializeStack (struct ThreadControl *t)
{
	unsigned int *top = t->stacktop;

	top--; *top = 0xABC0000E;	// LR: Return address
	top--; *top = (unsigned int) t->stacktop; // R13/SP
	top--; *top = 0xABC0000C;	// R12
	top--; *top = 0xABC0000B;	// R11
	top--; *top = 0xABC0000A;	// R10
	top--; *top = 0xABC00009;	// R9
	top--; *top = 0xABC00008;	// R8
	top--; *top = 0xABC00007;	// R7
	top--; *top = 0xABC00006;	// R6
	top--; *top = 0xABC00005;	// R5
	top--; *top = 0xABC00004;	// R4
	top--; *top = 0xABC00003;	// R3
	top--; *top = t->param3;	// R2: Normally environ
	top--; *top = t->param2;	// R1: Normally argv
	top--; *top = t->param1;	// R0: Normally argc
	top--; *top = (unsigned int) t->entrypoint + 4; // PC: Set to entry point
	top--; *top = (1<<5)|(1<<9)|(0x10);// CPSR: Set the thumb bit, user mode

	if (!t->privileged)
		top[0] |= 0x1F;

	t->stacktop = top;
}

void SchedSignalStackSetup (struct ThreadControl *thc)
{
	unsigned int *top = thc->stacktop;

	thc->stackstored = top;

	top--; *top = (unsigned int) thc->sigt; // LR: Return address
	top--; *top = (unsigned int) thc->stacktop; // R13/SP
	top--; *top = 0xDEF0000C;	// R12
	top--; *top = 0xDEF0000B;	// R11
	top--; *top = 0xDEF0000A;	// R10
	top--; *top = 0xDEF00009;	// R9
	top--; *top = 0xDEF00008;	// R8
	top--; *top = 0xDEF00007;	// R7
	top--; *top = 0xDEF00006;	// R6
	top--; *top = 0xDEF00005;	// R5
	top--; *top = 0xDEF00004;	// R4
	top--; *top = 0xDEF00003;	// R3
	top--; *top = 0xDEF00002;	// R2
	top--; *top = 0xDEF00001;	// R1
	top--; *top = thc->param1;	// R0
	top--; *top = (unsigned int) thc->sigh + 4; // PC: Set to entry point
	top--; *top = (1<<5)|(1<<9)|(0x10);// CPSR: Set the thumb bit, user mode

	if (!thc->privileged)
		top[0] |= 0x1F;

	thc->stacktop = top;
	thc->sigstate = 0;

	// Clear the signal pending bit
	thc->sigpending &= ~(1 << (thc->param1 - 1));
}

void SchedYield (void)
{
	SYS->SSIR1 = 0x00007575;
	__asm volatile("dsb");
	__asm volatile("isb");
}
