/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * This is where the CPU starts executing when it encounters an SVC or SWI
 * instruction. It essentially just runs the correct function in the dispatch
 * table.
 */

.section .text
.syntax unified
.cpu cortex-r4
.arm

.weak SVCHandler
.type SVCHandler, %function
SVCHandler:
	stmdb sp!, {r0-r3,r12,lr}		@ Stack regs that will be overwritten

	ldrb r12, [lr, #-1]				@ Load argument to SVC
	and r12, r12, #0x3F				@ Make sure to keep index within jump table bounds
	ldr lr, =svcJumpTable			@ Load address of jump table
	ldr r12, [lr, r12, lsl #2]		@ Load entry by shifting argument two times

	ldr r0, =currentthc				@ Load address of current thread
	ldr r0, [r0]					@ Load the actual pointer
	mov r1, sp						@ Store pointer to args

	blx r12							@ Branch and link to entry
	str r0, [sp]					@ Store return value

	ldmia sp!, {r0-r3,r12,lr}		@ Unstack
	movs pc, lr						@ Return, no subs for svc handler
.size SVCHandler, .-SVCHandler
