/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

.syntax unified
.cpu cortex-m3
.fpu softvfp
.thumb

.section .text.SchedContext
.global SchedContext
.type SchedContext, %function
SchedContext:					@ r0-r3, r12, lr, pc, xPSR already stacked
	mrs r0, psp					@ Fetch the thread stack pointer

	stmdb r0!, {r4-r11, r14}	@ Stack core regs, and save the stack
	ldr	r3, =currentthc
	ldr	r2, [r3]
	str r0, [r2]

.global SchedContextNext
SchedContextNext:
	bl SchedSelectNext			@ Call scheduler selection function

	ldr	r3, =currentthc			@ r0 becomes the stacktop member of currentthc
	ldr r1, [r3]
	ldr r0, [r1]

	ldr r2, [r1, #4]			@ Use correct privilege level
	msr control, r2

	ldr r3, =0xe000ed9c			@ Get address of the MPU_RBAR register
	add r1, r1, #8				@ Point r1 to our data in ThreadControl
	tst r2, #0x01				@ If unprivileged, set mpu regs
	itt ne
	ldmne r1, {r4-r11}			@ Copy using the alias registers
	stmne r3, {r4-r11}

	ldmia r0!, {r4-r11, r14}	@ Unstack core regs

	msr psp, r0					@ Set stack pointer and return
	bx r14						@ Unstack r0-r3, r12, lr, pc, xPSR
.size SchedContext, .-SchedContext
