/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

.syntax unified
.cpu cortex-r4
.arm

.section .text.UnknownHandler
.global UnknownHandler
.type UnknownHandler, %function
UnknownHandler:
	b UnknownHandler
.size UnknownHandler, .-UnknownHandler

/*
 * The tms570 is old-school, uses branch instructions instead of vector table.
 * See pdf "Interrupt and Exception Handling on Hercules(tm) ARM(R) Cortex(R)-R4/5-Based Microcontrollers"
 * (SPNA218) Table 3
 */

.section .vectortable.VectorTable,"a",%progbits
.global VectorTable
.type VectorTable, %object
.size VectorTable, .-VectorTable
VectorTable:
	VectorTableReset:	b KernelResetEntry	@ Reset, mode = Supervisor
	VectorTableUndef:	b UndefinedHandler	@ Undefined instruction, mode = "Undefined" mode
	VectorTableSVC:		b SVCHandler		@ SVC, mode = Supervisor
	VectorTablePABT:	b PrefetchAbort		@ Abort, mode = Abort
	VectorTableDABT:	b DataAbort			@ Abort, mode = Abort
	VectorTableRes:		b UnknownHandler	@ Reserved
	VectorTableIRQ:		ldr pc,[pc,#-0x1B0]	@ IRQ, mode = IRQ, see 15.7.2 in the manual
	VectorTableFIQ:		ldr pc,[pc,#-0x1B0]	@ FIQ, mode = FIQ, see 15.7.2 in the manual
