/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "drivers/dma/dma-stm32f103.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void DMAInit (DMAHandle *hdma)
{
	static const char bitshift[] = {0, 4, 8, 12, 16, 20, 24};

	unsigned int ch = ((unsigned int)hdma->chinst) & 0x0FF;
	ch = (ch - 0x0008) / 20;

	hdma->shift = bitshift[ch];
	hdma->dma = (DMA*) (((unsigned int)hdma->chinst) & 0xFFFFFF00);
}

void DMAStart (DMAHandle *hdma, void *src, void *dst, unsigned int nbytes)
{
	// Make sure the channel is disabled
	while (hdma->chinst->CR) hdma->chinst->CR = 0;

	// Set configuration
	hdma->chinst->CR = hdma->config;
	hdma->chinst->NDTR = nbytes;

	// Set destination and source
	if ((hdma->config & DMA_CR_DIR) == DMA_CR_DIR_M2P)
	{
		hdma->chinst->PAR = (unsigned int) dst;
		hdma->chinst->M0AR = (unsigned int) src;
	}
	else
	{
		hdma->chinst->PAR = (unsigned int) src;
		hdma->chinst->M0AR = (unsigned int) dst;
	}

	// Clear all pending interrupts
	hdma->dma->IFCR = (DMA_TEIF | DMA_HTIF | DMA_TCIF) << hdma->shift;

	// Enable interrupts
	hdma->chinst->CR |= DMA_CR_TCIE | DMA_CR_HTIE | DMA_CR_TEIE;

	// Enable DMA channel
	hdma->chinst->CR |= DMA_CR_EN;
}

void DMAIRQHandler (DMAHandle *hdma)
{
	// Transfer error
	if (hdma->dma->ISR & (DMA_TEIF << hdma->shift))
	{
		hdma->chinst->CR &= ~DMA_CR_TEIE;
		hdma->dma->IFCR = DMA_TEIF << hdma->shift;
	}

	// Half transfer complete
	if (hdma->dma->ISR & (DMA_HTIF << hdma->shift))
	{
		if (!(hdma->chinst->CR & DMA_CR_CIRC))
			hdma->chinst->CR &= ~DMA_CR_HTIE;

		hdma->dma->IFCR = DMA_HTIF << hdma->shift;
	}

	// Transfer complete
	if (hdma->dma->ISR & (DMA_TCIF << hdma->shift))
	{
		if (!(hdma->chinst->CR & DMA_CR_CIRC))
		{
			hdma->chinst->CR &= ~DMA_CR_TCIE;
			hdma->busystate = 0;
		}

		hdma->dma->IFCR = DMA_TCIF << hdma->shift;
	}
}
