/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "arch/tms570ls1224/dma.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void DMAInitModule (void)
{
	DMA->GCTRL |= 0x00010300;
	DMA->PAR[0] = 0x44444444;
	DMA->PAR[1] = 0x44444444;
}

void DMAChannelInit (int ch, int req,
	unsigned int src, unsigned int dst, unsigned int len,
	unsigned int args)
{
	// Assign DMA channel to request
	int chreg = ch >> 2;
	int chpos = (3 - (ch & 3)) << 3;
	unsigned int mod = ~(0x000000FF << chpos);

	DMA->DREQASI[chreg] = (DMA->DREQASI[chreg] & mod) | (req << chpos);

	// Set control "packet"
	DMACH->PCP[ch].ISADDR = src;
	DMACH->PCP[ch].IDADDR = dst;
	DMACH->PCP[ch].ITCOUNT = len << 16 | 1;
	DMACH->PCP[ch].CHCTRL = args;
	DMACH->PCP[ch].EIOFF = 0;
	DMACH->PCP[ch].FIOFF = 0;

	// Trigger the channel on HW request
	DMA->HWCHENAS |= 1 << ch;
}
