/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "drivers/gpio/gpio-stm32f407.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void GPIOInit (GPIO *gpio, int pins, int mode)
{
	for (int pin = 1, pinnum = 0; pinnum < 16; pin = pin << 1, pinnum++)
	{
		// Skip unspecified pin
		if (!(pins & pin))
			continue;

		// Disable while config
		gpio->MODER &= ~(0x3 << (pinnum << 1));

		// Set AF
		unsigned int af = (mode >> 16) & 0x0F;

		if (pinnum >= 8)
		{
			unsigned int shifter = ((pinnum - 8) << 2);
			gpio->AFRH &= ~(0x0F << shifter);
			gpio->AFRH |= af << shifter;
		}
		else
		{
			unsigned int shifter = pinnum << 2;
			gpio->AFRL &= ~(0x0F << shifter);
			gpio->AFRL |= af << shifter;
		}

		// Set pullup/down
		gpio->PUPDR &= ~(0x3 << (pinnum << 1));
		gpio->PUPDR |= ((mode >> 12) & 0x3) << (pinnum << 1);

		// Set speed
		gpio->OSPEEDR &= ~(0x3 << (pinnum << 1));
		gpio->OSPEEDR |= ((mode >> 8) & 0x3) << (pinnum << 1);

		// Set push-pull/open-drain
		gpio->OTYPER &= ~(1 << pinnum);
		gpio->OTYPER |= ((mode >> 4) & 1) << pinnum;

		// Set mode
		gpio->MODER |= (mode & 3) << (pinnum << 1);
	}
}
