/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"
#include "ioctl/ioctl.h"
#include "drivers/i2c/i2c-stm32f407.h"

///////////////////////////////////////////////////////////////////////////////
// Driver functions
///////////////////////////////////////////////////////////////////////////////
int dev_i2c_open (struct FileNode *node, const char *filename)
{
	I2CInit ();
	return 0;
}

int dev_i2c_close (struct FileNode *node)
{
	return 0;
}

int dev_i2c_read (struct FileNode *node, void *data, unsigned int nbytes)
{
	unsigned char addr = ((unsigned char*)data)[0];

	if (I2CReadPacket (addr, data+1, nbytes-1))
		return -EIO;
	else
		return nbytes;
}

int dev_i2c_write (struct FileNode *node, void *data, unsigned int nbytes)
{
	unsigned char addr = ((unsigned char*)data)[0];

	if (I2CWritePacket (addr, data+1, nbytes-1))
		return -EIO;
	else
		return nbytes;
}

int dev_i2c_ioctl (struct FileNode *node, unsigned int code, void *data)
{
	switch (code)
	{
	default:
		// Unknown ioctl: Return "Inappropriate ioctl for device"
		return -ENOTTY;
	}

	// Default return value
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Driver declarations
///////////////////////////////////////////////////////////////////////////////
MODULE_DESCRIPTOR (i2c,
	dev_i2c_open,
	dev_i2c_close,
	dev_i2c_read,
	dev_i2c_write,
	dev_i2c_ioctl,
	"i2c");
