/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"
#include "mem/string.h"
#include "print/print.h"
#include "ioctl/ioctl.h"
#include "platform/platform.h"
#include "drivers/lcd/lcd.h"
#include "../config/live/drivers/lcd.h"

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////
extern int lcdWidth;
extern int lcdHeight;
char lcdOrientation;
char lcdInitialized;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void LCDInit (void)
{
	// ILI9488

	if (lcdInitialized)
		return;

	lcdInitialized = 1;

	LCD_COMMAND (0x11);

	unsigned int start = PlatformGetTick ();
	while ((PlatformGetTick () - start) < 120);

	LCD_COMMAND (0xE0);
	LCD_DATA (0x00);
	LCD_DATA (0x03);
	LCD_DATA (0x09);
	LCD_DATA (0x08);
	LCD_DATA (0x16);
	LCD_DATA (0x0A);
	LCD_DATA (0x3F);
	LCD_DATA (0x78);
	LCD_DATA (0x4C);
	LCD_DATA (0x09);
	LCD_DATA (0x0A);
	LCD_DATA (0x08);
	LCD_DATA (0x16);
	LCD_DATA (0x1A);
	LCD_DATA (0x0F);

	LCD_COMMAND (0xE1);
	LCD_DATA (0x00);
	LCD_DATA (0x16);
	LCD_DATA (0x19);
	LCD_DATA (0x03);
	LCD_DATA (0x0F);
	LCD_DATA (0x05);
	LCD_DATA (0x32);
	LCD_DATA (0x45);
	LCD_DATA (0x46);
	LCD_DATA (0x04);
	LCD_DATA (0x0E);
	LCD_DATA (0x0D);
	LCD_DATA (0x35);
	LCD_DATA (0x37);
	LCD_DATA (0x0F);

	LCD_COMMAND (0xC0);
	LCD_DATA (0x17);
	LCD_DATA (0x15);

	LCD_COMMAND (0xC1);
	LCD_DATA (0x41);

	LCD_COMMAND (0xC5);
	LCD_DATA (0x00);
	LCD_DATA (0x12);
	LCD_DATA (0x80);

	LCD_COMMAND (0x36);
	LCD_DATA (0x48);

	LCD_COMMAND (0x3A);
	LCD_DATA (0x55);

	LCD_COMMAND (0xB0);
	LCD_DATA (0x00);

	LCD_COMMAND (0xB1);
	LCD_DATA (0xA0);

	LCD_COMMAND (0xB4);
	LCD_DATA (0x02);

	LCD_COMMAND (0xB6);
	LCD_DATA (0x02);
	LCD_DATA (0x02);

	LCD_COMMAND (0xE9);
	LCD_DATA (0x00);

	LCD_COMMAND (0xF7);
	LCD_DATA (0xA9);
	LCD_DATA (0x51);
	LCD_DATA (0x2C);
	LCD_DATA (0x82);

	LCD_COMMAND (0x11);
	start = PlatformGetTick ();
	while ((PlatformGetTick () - start) < 120);

	LCDRotate (LCD_DEFAULT_ORIENTATION);

	LCD_COMMAND (0x29);
}


void LCDRotate (int direction)
{
	if (direction == LCD_VERTICAL)
	{
		// Vertical
		LCD_COMMAND (0x36);
		LCD_DATA (0x48);
	}
	else
	{
		// Horizontal
		LCD_COMMAND (0x36);
		LCD_DATA (0x80 | 0x40 | 0x20 | 0x08);
	}

	lcdOrientation = direction;
}

void LCDSetAddress (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2)
{
	LCD_COMMAND (0x2A);
	LCD_DATA (x1 >> 8);
	LCD_DATA (x1 & 0x00FF);
	LCD_DATA (x2 >> 8);
	LCD_DATA (x2 & 0x00FF);
	LCD_COMMAND (0x2B);
	LCD_DATA (y1 >> 8);
	LCD_DATA (y1 & 0x00FF);
	LCD_DATA (y2 >> 8);
	LCD_DATA (y2 & 0x00FF);
	LCD_COMMAND (0x2C);
}
