/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * Crappy devnode for getting stats about network stuffs.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"
#include "ioctl/ioctl.h"
#include "mem/string.h"

#include "../config/live/network-config.h"
#include "net/enet/network.h"
#include "net/enet/layer4/udp.h"
#include "net/enet/layer2/linkspeed.h"

///////////////////////////////////////////////////////////////////////////////
// Vars
///////////////////////////////////////////////////////////////////////////////
extern struct ENetGlobals eglobals;

///////////////////////////////////////////////////////////////////////////////
// Driver functions
///////////////////////////////////////////////////////////////////////////////
int dev_net_open (struct FileNode *node, const char *filename)
{
	return 0;
}

int dev_net_close (struct FileNode *node)
{
	return 0;
}

int dev_net_read (struct FileNode *node, void *data, unsigned int nbytes)
{
	// Find the interface
	Layer2 *interface = eglobals.iflist;
	int n = (int) node->priv;

	for (int i = 0; i < n && interface; i++)
		interface = interface->next;

	// Copy if enough space
	if (interface && nbytes >= sizeof (struct Layer2Statistics))
	{
		MemCpy (data, &interface->stats, sizeof (struct Layer2Statistics));
		return sizeof (struct Layer2Statistics);
	}

	return -1;
}

int dev_net_write (struct FileNode *node, void *data, unsigned int nbytes)
{
	// Set index
	if (nbytes > 0)
		node->priv = (void*) ((unsigned int) ((unsigned char*)data)[0]);

	// Find the interface
	Layer2 *interface = eglobals.iflist;
	int n = (int) node->priv;

	for (int i = 0; i < n && interface; i++)
		interface = interface->next;

	// Get connected state
	if (interface)
		return Layer2LinkUp (interface);

	return -1;
}

int dev_net_ioctl (struct FileNode *node, unsigned int code, void *data)
{
	switch (code)
	{
	default:
		// Unknown ioctl: Return "Inappropriate ioctl for device"
		return -ENOTTY;
	}

	// Default return value
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Driver declarations
///////////////////////////////////////////////////////////////////////////////
MODULE_DESCRIPTOR (net,
	dev_net_open,
	dev_net_close,
	dev_net_read,
	dev_net_write,
	dev_net_ioctl,
	"net");
