/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"
#include "ioctl/ioctl.h"

///////////////////////////////////////////////////////////////////////////////
// Driver functions
///////////////////////////////////////////////////////////////////////////////
int dev_null_open (struct FileNode *node, const char *filename)
{
	return 0;
}

int dev_null_close (struct FileNode *node)
{
	return 0;
}

int dev_null_read (struct FileNode *node, void *data, unsigned int nbytes)
{
	return 0;
}

int dev_null_write (struct FileNode *node, void *data, unsigned int nbytes)
{
	return nbytes;
}

int dev_null_ioctl (struct FileNode *node, unsigned int code, void *data)
{
	switch (code)
	{
	default:
		// Unknown ioctl: Return "Inappropriate ioctl for device"
		return -ENOTTY;
	}

	// Default return value
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Driver declarations
///////////////////////////////////////////////////////////////////////////////
MODULE_DESCRIPTOR (null,
	dev_null_open,
	dev_null_close,
	dev_null_read,
	dev_null_write,
	dev_null_ioctl,
	"null");
