/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"
#include "mem/string.h"
#include "drivers/gpio/gpio-stm32f407.h"
#include "drivers/dma/dma-stm32f407.h"
#include "drivers/sdio/sdio-stm32f407.h"
#include "arch/stm32f407/rcc.h"
#include "platform/platform.h"
#include "ioctl/ioctl.h"
#include "print/print.h"

///////////////////////////////////////////////////////////////////////////////
// Private functions
///////////////////////////////////////////////////////////////////////////////
void dev_sdio_dumpresponse (sdio *t)
{
	Print ("SDIO RESP1 0x%x\n", t->instance->RESP1);
	Print ("SDIO RESP2 0x%x\n", t->instance->RESP2);
	Print ("SDIO RESP3 0x%x\n", t->instance->RESP3);
	Print ("SDIO RESP4 0x%x\n", t->instance->RESP4);
}

void dev_sdio_printcid (sdio *t)
{
	int mid = (t->cid[0] >> 24) & 0xFF;
	int oid = (t->cid[0] >> 8) & 0xFFFF;
	int pnm0 = t->cid[0] & 0xFF;
	int pnm1 = t->cid[1];
	int prv = (t->cid[2] >> 24) & 0xFF;
	int psn = ((t->cid[2] & 0x00FFFFFF) << 8) | (t->cid[3] >> 24);
	int mdt = (t->cid[3] >> 8) & 0x0FFF;

	Print ("Manufacturer ID (MID):    0x%x (%i)\n", mid, mid);
	Print ("OEM/Application ID (OID): 0x%x (%i)\n", oid, oid);
	Print ("Product name (PNM):       0x%x (%i)\n", pnm0, pnm0);
	Print ("Product name (PNM):       0x%x (%i)\n", pnm1, pnm1);
	Print ("Product revision (PRV):   0x%x (%i)\n", prv, prv);
	Print ("Product serial no (PSN):  0x%x (%i)\n", psn, psn);
	Print ("Manufacturing date (MDT): 0x%x (%i)\n", mdt, mdt);
}

void dev_sdio_printcsd (sdio *t)
{
	int csd_structure = t->csd[0] >> 30;
	int c_size = ((t->csd[1] & 0x3F) << 16) | ((t->csd[2] >> 16) & 0x0000FFFF);

	Print ("CSD Structure 0x%x (%i)\n", csd_structure, csd_structure);
	Print ("Device Size 0x%x (%i)\n", c_size, c_size);
}
