/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"
#include "mem/string.h"
#include "print/print.h"
#include "ioctl/ioctl.h"
#include "arch/tms570ls1224/platform.h"

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////
extern char loader_printrel[16];

///////////////////////////////////////////////////////////////////////////////
// Driver functions
///////////////////////////////////////////////////////////////////////////////
int dev_sys_open (struct FileNode *node, const char *filename)
{
	node->priv = 0;
	return 0;
}

int dev_sys_close (struct FileNode *node)
{
	return 0;
}

int dev_sys_read (struct FileNode *node, void *data, unsigned int nbytes)
{
	int which = (int) node->priv;
	const char *str = "";

	if (which == 0)
		str = VERSION_GITCOMMIT;
	else if (which == 1)
		str = VERSION_DATE;

	int n = StrLen (str);

	if (nbytes < n)
		n = nbytes;
	else
		node->priv = (void*) 0xFFFFFFFF;

	MemCpy (data, (char*) str, n);
	return n;
}

int dev_sys_write (struct FileNode *node, void *data, unsigned int nbytes)
{
	if (nbytes >= 4)
	{
		unsigned int *d = (unsigned int*) data;
		node->priv = (void*) d[0];
	}

	return nbytes;
}

int dev_sys_ioctl (struct FileNode *node, unsigned int code, void *data)
{
	switch (code)
	{
	case SYSSHOWREL:
		if (!data)
			loader_printrel[0] = 0;
		else
		{
			char *str = (char*) data;

			for (int i = 0; i < sizeof (loader_printrel); i++)
				if (!(loader_printrel[i] = str[i]))
					break;

			loader_printrel[sizeof (loader_printrel) - 1] = 0;
		}

		break;
	case SYSREBOOT:
		SYS->SYSECR = 0x00008000;
		break;
	default:
		// Unknown ioctl: Return "Inappropriate ioctl for device"
		return -ENOTTY;
	}

	// Default return value
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Driver declarations
///////////////////////////////////////////////////////////////////////////////
MODULE_DESCRIPTOR (sys,
	dev_sys_open,
	dev_sys_close,
	dev_sys_read,
	dev_sys_write,
	dev_sys_ioctl,
	"sys");
