/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "mem/string.h"
#include "posix/errno.h"
#include "fs/dev/dev.h"
#include "ioctl/ioctl.h"
#include "posix/sys/signal.h"
#include "process/process.h"
#include "drivers/dma/dma-tms570.h"
#include "drivers/tty/tty-tms570.h"

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Interrupt handler
///////////////////////////////////////////////////////////////////////////////
void dev_tty_irq_direct_circ (tty *t)
{
	ProcessSendSignal (-t->pgid, SIGURG);
}

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int dev_tty_read_direct_circ (tty *t, void *data, unsigned int nbytes)
{
	// Fail if already in the receiving state, ignore error mode
	if (t->busystate == 1)
		return -EBUSY;

	// Reset DMA, stop SCI DMA transfer, disable tx
	DMA->BTCFLAG = 1 << t->dmachrx;
	t->instance->CLEARINT = 3 << 17;
	t->instance->GCR1 &= ~(1 << 24);
	t->busystate = 0;

	// Set receive DMA, circular
	DMAChannelInit (t->dmachrx, t->reqchrx,
		((unsigned int) &t->instance->RD) + 3,
		(unsigned int) data,
		nbytes,
		3);

	// Static arrays used to trick the arbiter
	static const unsigned char bullshitarray[8] = {0, 1, 2, 3, 4, 5, 6, 7};
	static unsigned char bullshitcounter = 0;

	// Set bullshit DMA
	DMAChannelInit (t->dmachbs, t->reqchrx,
		(unsigned int) bullshitarray,
		(unsigned int) &bullshitcounter,
		1,
		9);

	// Set busy state
	t->busystate = 1;

	// Enable transfer
	t->instance->GCR1 |= 1 << 24;
	t->instance->SETINT = 3 << 17;

	// Bullshit DMA IRQ enable
	DMA->GCHIENAS = 1 << t->dmachbs;
	DMA->BTCINTENAS = 1 << t->dmachbs;

	// Done
	return nbytes;
}
