/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

.syntax unified
.cpu cortex-m3
.fpu softvfp
.thumb

/*
 * r0 = data
 * r1 = nbytes
 * r2 = bsrr
 * r3 = pin
 * r4 = brr
 * r5 = inner and variable 0x80
 * r6 = data byte
 */

.section .text
.global WS2811OutLow
.type WS2811OutLow, %function
WS2811OutLow:
	// 0.25 us / 1.25 divs
	strh r3, [r2]
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	// 1 us / 5 divs (total 6.25 divs)
	strh r3, [r4]
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bx lr
.size WS2811OutLow, .-WS2811OutLow

.section .text
.global WS2811OutHigh
.type WS2811OutHigh, %function
WS2811OutHigh:
	// 0.6 us / 3 divs
	strh r3, [r2]
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	// 0.65 / 3.25 divs
	strh r3, [r4]
	nop
	nop
	nop
	nop
	nop
	nop
	bx lr
.size WS2811OutHigh, .-WS2811OutHigh

.section .text
.global WS2811OutAsm
.type WS2811OutAsm, %function
WS2811OutAsm:
	// Stack regs
	push {r4, r5, r6, r7, lr}

	// Prepare BRR pointer
	add r4, r2, #4

	// Outer loop: for (; data != end; data++)
	add r1, r0, r1
outer:

	// Inner: for (j = 0x80, byte = data[0]; j; j = j >> 1)
	ldr r5, =0x80
	ldrb r6, [r0]
inner:
	and r7, r6, r5
	cmp r7, #0
	bne highbit
	// Low bit
		bl WS2811OutLow
		b donebit
	// High bit
	highbit:
		bl WS2811OutHigh
	donebit:

	// Inner control
	lsrs r5, r5, #1
	cmp r5, #0
	bne inner

	// Outer control: data++
	add r0, #1
	cmp r0, r1
	bne outer

	// Return
	pop {r4, r5, r6, r7, lr}
	bx lr
.size WS2811OutAsm, .-WS2811OutAsm
