/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/pipe/pipe-internal.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
pipe_data *vfs_pipe_allocate (pipe_data **root, const char *name)
{
	// Set interrupted call
	vfs_interrupted = 1;

	// Get size of name
	int namelen = StrLen (name);

	// Calculate needed size
	int size = sizeof (pipe_data) - PIPE_NAME_MAXSIZE + namelen + 1;

	// Get a new one
	pipe_data *p = (struct pipe_data*) MemAlloc (size, 0);

	// If valid
	if (p)
	{
		// Set as root if first, otherwise add to tail
		if (*root == 0)
			*root = p;
		else
		{
			pipe_data *tail = *root;
			while (tail->next) tail = tail->next;
			tail->next = p;
		}

		// Clear data structure and copy name
		MemSet (p, 0, size);
		StrCpy (p->name, name);
	}

	// Return the pipe
	return p;
}

void vfs_pipe_free (pipe_data **root, pipe_data *p)
{
	vfs_interrupted = 1;

	if (*root == p)
		*root = p->next;
	else
	{
		pipe_data *tail = *root;
		while (tail->next != p) tail = tail->next;
		tail->next = p->next;
	}

	MemFree ((unsigned int) p);
}
