/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "fs/pipe/pipe-internal.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int vfs_pipe_read_nonblocking (pipe_data *p, void *data, int nbytes)
{
	int n = p->nbytesinpipe;

	if (nbytes < n)
		n = nbytes;

	for (int i = 0; i < n; i++)
		((char*)data)[i] = p->contents[(p->readpos + i) & PIPE_BUF_MASK];

	p->readpos = (p->readpos + n) & PIPE_BUF_MASK;
	p->nbytesinpipe -= n;

	return n;
}

int vfs_pipe_write_nonblocking (pipe_data *p, void *data, int nbytes)
{
	int n = PIPE_BUF_SIZE - p->nbytesinpipe;

	if ((p->flags & PIPE_FLAG_MSG) && (n < nbytes))
		return 0;

	if (nbytes < n)
		n = nbytes;

	for (int i = 0; i < n; i++)
		p->contents[(p->writepos + i) & PIPE_BUF_MASK] = ((char*)data)[i];

	p->writepos = (p->writepos + n) & PIPE_BUF_MASK;
	p->nbytesinpipe += n;

	return n;
}
