/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "mem/mem.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
unsigned short MemTestBlock (unsigned int start, unsigned int end, unsigned short *testarr)
{
	for (unsigned int i = start; i < end; i += 512)
	{
		volatile unsigned short *r = (volatile unsigned short*) i;

		for (unsigned int data = 0; data < 256; data++)
			r[data] = testarr[data];

		for (unsigned int data = 0; data < 256; data++)
		{
			if (r[data] != testarr[data])
				return r[data] ^ testarr[data];
		}
	}

	return 0;
}

int MemTest (unsigned int addr, unsigned int nbytes)
{
	unsigned short testarr[256];
	unsigned int div = nbytes / 16;

	for (unsigned int i = 0; i < 256; i++)
		testarr[i] = (~i & 0x00FF) | (i << 8);

	for (unsigned int s = 0; s < 16; s++)
	{
		unsigned int start = addr + (s * div);
		unsigned int end = addr + ((s + 1) * div);

		unsigned short result = MemTestBlock (start, end, testarr);

		if (result)
			return 0;
	}

	return 1;
}
