/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void StrCpy (char *dst, const char *src)
{
	while (*src)
	{
		*dst++ = *src++;
	}

	*dst = 0;
}

void StrCat (char *dst, const char *src)
{
	while (*dst) dst++;

	StrCpy (dst, src);
}

void MemSet (void *dst, int val, unsigned int nbytes)
{
	for (unsigned int i = 0; i < nbytes; i++)
		((unsigned char*)dst)[i] = (unsigned char) val;
}

void *memset (void *dst, int val, unsigned int nbytes)
{
	MemSet (dst, val, nbytes);
	return dst;
}

void MemCpy (void *dst, void *src, unsigned int nbytes)
{
	for (unsigned int i = 0; i < nbytes; i++)
		((unsigned char*)dst)[i] = ((unsigned char*)src)[i];
}

int memcpy (void *dst, void *src, unsigned int nbytes)
{
	MemCpy (dst, src, nbytes);
	return nbytes;
}

int MemCmp (const void *dst, const void *src, unsigned int nbytes)
{
	const char *a = (const char*) dst;
	const char *b = (const char*) src;

	for (unsigned int i = 0; i < nbytes; i++)
	{
		if (a[i] < b[i])
			return -1;
		else if (a[i] > b[i])
			return 1;
	}

	return 0;
}

int StrCmp (const char *a, const char *b)
{
	while (*a == *b && *a)
	{
		a++;
		b++;
	}

	if (!*a && !*b)
		return 0;
	else if (*a < *b)
		return -1;
	else
		return 1;
}

int StrLen (const char *s)
{
	int l = 0;
	while (*s++) l++;
	return l;
}

int strlen (const char *s)
{
	return StrLen (s);
}

void Reverse (char *s)
{
	int l = StrLen (s);

	for (int i = 0, j = l-1; i < j; i++, j--)
	{
		char c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}

char *strtok (char *in, char *match)
{
	static char *current;

	if (in)
		current = in;

	char *out = current;

	while (*current && *current != *match)
		current++;

	if (*current == *match)
	{
		*current = 0;
		current++;
		return out;
	}
	else
		return out;
}

int atoi (char *in)
{
	Reverse (in);

	int len = StrLen (in);
	int ret = 0;
	int po = 1;

	for (int i = 0; i < len; i++)
	{
		if (in[i] == '-')
			return -ret;
		else
			ret += (in[i] - '0') * po;

		po *= 10;
	}

	return ret;
}
