/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "net/enet/layer2/checksum.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
unsigned char ChecksumCalculate (DataFrame *f)
{
	unsigned int *iptr = (unsigned int*) f;
	unsigned char oldsum = f->checksum;
	unsigned int sum = 0;
	int n = f->size / 4;
	int r = f->size & 3;

	f->checksum = ' ';

	for (int i = 0; i < n; i++, iptr++)
		sum += *iptr;

	unsigned char *cptr = (unsigned char*) iptr;

	for (int i = 0; i < r; i++, cptr++)
		sum += (unsigned char) *cptr;

	f->checksum = oldsum;

	sum &= 0x000000FF;

	if (sum >= 254)
		sum -= 254;

	return (unsigned char) sum;
}

int ChecksumVerify (DataFrame *f)
{
#if 1
	if (f->checksum == ' ')
		return 2;
#endif

	unsigned char calculated = ChecksumCalculate (f);

	if (calculated != f->checksum)
		return 1;

	return 0;
}
