/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by erlendjs on 14/04/16.
//

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "net/enet/layer2/framepool.h"
#include "../config/live/network-config.h"

#if NETWORK_DEBUG_FRAMEPOOL
#include NETWORK_DEBUG_INC
#endif

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#if NETWORK_DEBUG_FRAMEPOOL
	#define FRAMEPOOLDEBUG(str) str
#else
	#define FRAMEPOOLDEBUG(str)
#endif

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////
DataFrame framepool[FRAMEPOOL_POOLSIZE];
unsigned char usage[FRAMEPOOL_POOLSIZE];

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void FramepoolInit (void)
{
	// Loop through slots and clear the usage flag
	for (int i = 0; i < FRAMEPOOL_POOLSIZE; i++)
		usage[i] = 0;
}

DataFrame *FramepoolNew (void)
{
	// Loop through slots to find a free one
	for (int i = 0; i < FRAMEPOOL_POOLSIZE; i++)
	{
		// Check flag
		if (!usage[i])
		{
			// Set flag
			usage[i] = 1;

			// Print debug message
			FRAMEPOOLDEBUG (NETWORK_PRINTI ("FP Allocating %i\n", i));

			// Clear the contents if enabled
#if FRAMEPOOL_ERASE
			for (int j = 0; j < 256+4; j++)
				((unsigned char*)&framepool[i])[j] = 0xFA;
#endif

			// Return its pointer
			return &framepool[i];
		}
	}

	// If there are no free slots, print message and return 0
	FRAMEPOOLDEBUG (NETWORK_PRINTS ("FP Allocation failed\n"));
	return 0;
}

void FramepoolDelete (DataFrame *frame)
{
	// Recalculate the index
#if __LP64__ == 1
	unsigned int i = (((unsigned long long) frame - (unsigned long long) framepool) / sizeof (DataFrame));
#else
	unsigned int i = (((unsigned int) frame - (unsigned int) framepool) / sizeof (DataFrame));
#endif

	// Print debug message
	FRAMEPOOLDEBUG (NETWORK_PRINTI ("FP Deleting %i\n", i));

	// Clear usage flag
	usage[i] = 0;
}
