/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by erlendjs on 10/05/16.
//

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "platform/platform.h"
#include "net/enet/network.h"
#include "net/enet/layer2/arp.h"
#include "net/enet/layer2/switch.h"
#include "net/enet/layer2/framepool.h"
#include "net/enet/layer2/manager.h"

///////////////////////////////////////////////////////////////////////////////
// Private variables
///////////////////////////////////////////////////////////////////////////////
struct ENetGlobals eglobals;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void NetInit (unsigned short myipaddress)
{
	// Initialize frame pool
	FramepoolInit ();

	// Initialize hardware drivers
	Layer2GlobalInit ();

	// Add interfaces
	for (struct Layer2 *i = eglobals.iflist; i; i = i->next)
		Layer2Init (i);

	// Initialize switch
	Layer2SwitchInit ();

	// Initialize ARP
	ARPInit (myipaddress);

	// Vars
	eglobals.myip = myipaddress;
	eglobals.defaultgateway = IP_DEFAULT_GATEWAY;
}

void NetPump (void)
{
	// Process interface data
	for (struct Layer2 *i = eglobals.iflist; i; i = i->next)
		Layer2Pump (i);

	// Process datagrams
	DataFrame *frame;

	while ((frame = Layer2GlobalPull ()))
	{
		// Kill null frames early, otherwise transfer to switch
		if (frame->destination == 0)
			FramepoolDelete (frame);
		else
			Layer2SwitchProcessFrame(frame);
	}

	// Call switch list maintenance every second
	static unsigned int start = 0;

	if ((PlatformGetTick () - start) > 1000)
	{
		start = PlatformGetTick ();
		Layer2SwitchMaintainList ();
	}
}

int NetLinkUp (void)
{
	// Check if at least one interface is up
	for (struct Layer2 *i = eglobals.iflist; i; i = i->next)
		if (Layer2LinkUp (i))
			return 1;

	// No interfaces up
	return 0;
}
