/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "syscall/svc-basicfileops.h"
#include "fs/vfs.h"
#include "fs/dev/dev.h"
#include "posix/errno.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int ProcessAllocateFileDesc (struct ThreadControl *ctc, struct FileNode **fnodeout)
{
	// Find a free file descriptor in the process
	int pfd = 255;

	for (int i = 0; i < MAX_NODES; i++)
		if (ctc->pctrl->fnodes[i] == 255)
			{pfd = i; break;}

	// Check if it is valid
	if (pfd == 255)
		return -ENOMEM;

	// It is valid, find a kernel file descriptor
	int kfd = -1;

	for (int i = 0; i < NODETABLE_SIZE; i++)
		if (!nodetable[i].usecounter)
			{kfd = i; break;}

	// Check if it is valid
	if (kfd < 0)
		return -ENOMEM;

	// Clear dev and flags
	nodetable[kfd].dev = 0;
	nodetable[kfd].flags = 0;
	nodetable[kfd].usecounter = 1;
	nodetable[kfd].fd = pfd;

	// Success, set the allocation
	ctc->pctrl->fnodes[pfd] = kfd;
	*fnodeout = &nodetable[kfd];
	return pfd;
}
