/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "posix/sys/types.h"
#include "posix/spawn.h"
#include "ioctl/ioctl.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Local functions
///////////////////////////////////////////////////////////////////////////////
__attribute__((naked))
int asm_posix_spawn (pid_t *pid, const char *name, int *arg, int *errno)
{
	asm volatile("svc 31\n"
				"bx lr\n"
				"nop");
}

__attribute__((naked))
int asm_open (const char *name, int mode, int asdf)
{
	asm volatile ("svc 0\n"
		"bx lr\n");
}

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int open (const char *name, int mode, ...)
{
	return asm_open (name, mode, 0);
}

__attribute__((naked))
int close (int fd)
{
	asm volatile ("svc 1\n"
		"bx lr\n");
}

__attribute__((naked))
int read (int fd)
{
	asm volatile ("svc 2\n"
		"bx lr\n");
}

__attribute__((naked))
int syscall_write (int fd, const void *data, int nbytes, int *error)
{
	asm volatile("svc 3\n"
				"bx lr\n"
				"nop");
}

__attribute__((naked))
int ioctl (int fd, int code, void *data)
{
	asm volatile ("svc 4\n"
		"bx lr\n");
}

__attribute__((naked))
off_t lseek (int fd, off_t offset, int whence)
{
	asm volatile ("svc 5\n"
		"bx lr\n");
}

__attribute__((naked))
int waitpid (int pid, int *status, int flags)
{
	asm volatile("svc 32\n"
				"bx lr\n"
				"nop");
}

int posix_spawn (pid_t *pid,
	const char *name,
	const posix_spawn_file_actions_t *factions,
	const posix_spawnattr_t *attr,
	char **argv,
	char **env)
{
	// Construct extra arguments
	int arg[4] = {(int)factions, (int)attr, (int)argv, (int)env};

	// Fake errno
	int errno = 0;

	// Make the assembly call
	return asm_posix_spawn (pid, name, arg, &errno);
}

int tcsetpgrp (int pfd, int pgid)
{
	// TODO: Ioctl
	return ioctl (pfd, TCSETPGRP, (void*)pgid);
}

ssize_t write (int a, const void *b, ssize_t c)
{
	int err = 0;
	return syscall_write (a, b, c, &err);
}
