/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "process/processcontrol.h"
#include "fs/vfs.h"
#include "posix/errno.h"
#include "posix/sys/stat.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// For manipulating stack arguments
#define RET ((int*)args)[0]
#define ERRN ((int**)args)[3]

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int svc_lseek (struct ThreadControl *ctc, void *args)
{
	// File descriptor index from the caller
	unsigned int pfd = ((unsigned int*)args)[0];

	// Index to file descriptor in the kernel
	int kfd;

	// Check for validity and get kfd
	if (pfd < MAX_NODES && (kfd = ctc->pctrl->fnodes[pfd]) != 255)
	{
		int ret = nodetable[kfd].fs->lseek (&nodetable[kfd], ((int*)args)[1], ((int*)args)[2]);

		if (ret < 0)
		{
			*ERRN = -ret;
			return -1;
		}

		return ret;
	}

	// Invalid descriptor
	*ERRN = EBADF;
	return -1;
}

int svc_fstat (struct ThreadControl *ctc, void *args)
{
	// File descriptor index from the caller
	unsigned int pfd = ((unsigned int*)args)[0];

	// Index to file descriptor in the kernel
	int kfd;

	// Check for validity and get kfd
	if (pfd < MAX_NODES && (kfd = ctc->pctrl->fnodes[pfd]) != 255)
	{
		int ret = nodetable[kfd].fs->fstat (&nodetable[kfd], (struct stat *)((int*)args)[1]);

		if (ret < 0)
		{
			*ERRN = -ret;
			return -1;
		}

		return ret;
	}

	// Invalid descriptor
	*ERRN = EBADF;
	return -1;
}
