/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "process/processcontrol.h"
#include "fs/vfs.h"
#include "posix/errno.h"
#include "posix/sys/stat.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// For manipulating stack arguments
#define ERRN ((int**)args)[3]

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int svc_mount (struct ThreadControl *ctc, void *args)
{
	/*
	 * TODO: In order to mount, we have to open the underlying device
	 * and identify the correct driver as things are now. This may be a
	 * bit tricky for file systems that do not actually need a device.
	 * In any case, we do not want the userspace application to be the
	 * owner of that device.
	 */
	(void)ctc;
	(void)args;
	return -1;
}

int svc_unmount (struct ThreadControl *ctc, void *args)
{
	// CTC Is not used!
	(void)ctc;

	// Get the path
	const char *path = (const char*)((int*)args)[0];

	// Do the statvfs operation
	int ret = vfs_unmount (path);

	// Check for errors and return
	if (ret < 0)
	{
		*ERRN = -ret;
		return -1;
	}

	return ret;
}
