/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef WORKER_H
#define WORKER_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "worker/worker.h"
#include "sched/scheduler.h"

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////
// TODO: Check required stack space
// The stack space here may seem excessive. It is caused by directly called
// open in the spawn attribute handling.
unsigned int workerstack[256+16+8];
struct ThreadControl workerthread;

///////////////////////////////////////////////////////////////////////////////
// Private functions
///////////////////////////////////////////////////////////////////////////////
void WorkerThread (void)
{
	while (1)
	{
		// Fetch work from queue, note that this function blocks until
		// work is available.
		struct ThreadControl *prc = SchedFetchWork ();

		// Call the work function
		// TODO Put the adder into a platform header
		if (!prc->callback (prc, ARCH_STACK_ACCESSOR(prc)))
		{
			// If not crashed:
			// Erase the function pointer so that SchedState does not call it
			prc->callback = 0;

			// Unblock the thread we did work for
			SchedState (prc, SCHED_STATUS_RUN, 0, SCHED_STATUS_DEFERRED, 0);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
void WorkerStart (struct ProcessControl *parent)
{
	// Privileged thread
	workerthread.privileged = 0;

	// Create and start worker thread
	workerthread.pctrl = parent;
	SchedAddThread (&workerthread, &WorkerThread, workerstack, 256+16+8, SCHED_PRIORITY_MEDIUM);
}

void WorkerUnblock (void)
{
	int crit = SchedEnterCritical ();

	if (workerthread.state & SCHED_STATUS_SLEEPING)
		SchedState (&workerthread, SCHED_STATUS_RUN, 0, SCHED_STATUS_SLEEPING, 0);

	SchedExitCritical (crit);
}

#endif
