/*
 * Copyright (c) 2018, Erlend Sveen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef FCNTL_H
#define FCNTL_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include <sys/ioctl.h>
#include <sys/stat.h>

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////

// fcntl commands
// Defined in sys/ioctl.h

// fcntl flags: file descriptor
#define FD_CLOEXEC		(0 << 0)	// Not supported

// fcntl flags: record locking
#define F_RDLCK			1			// Not supported
#define F_UNLCK			2			// Not supported
#define F_WRLCK			3			// Not supported

// open flags: file creation
#define O_CREAT         (1 << 9)
#define O_EXCL			(0 << 0)	// Not supported
#define O_NOCTTY		(0 << 0)	// Not supported
#define O_TRUNC         (1 << 10)

// open and fcntl flags: file status
#define O_APPEND        (1 << 3)
#define O_DSYNC			(0 << 0)	// Not supported
#define O_NONBLOCK		(1 << 4)	// Not supported
#define O_RSYNC			(0 << 0)	// Not supported
#define O_SYNC			(0 << 0)	// Not supported

// mask for use with file access modes
#define O_ACCMODE		(0 << 0)	// Not supported

// open file access modes
#define O_RDONLY        (0 << 0)
#define O_RDWR          (1 << 1)
#define O_WRONLY        (1 << 0)

// ??
#define FASYNC			0x1000

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int creat (const char *, mode_t);	// Not supported
int fcntl (int, int, ...);
int open (const char*, int, ...);
int posix_fadvise (int, off_t, off_t, int); // Not supported
int posix_fallocate (int, off_t, off_t); // Not supported

#endif
