/*
 * Copyright (c) 2018, Erlend Sveen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SPAWN_H
#define SPAWN_H

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "sys/types.h"

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define POSIX_SPAWN_SETPGROUP 1
#define POSIX_SPAWN_NFACTIONS 2

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
typedef struct posix_spawnattr_t
{
	int flags;
	int pgroup;
}posix_spawnattr_t;

typedef struct file_action
{
	int fildes;
	char *path;
	int oflag;
	int mode;
	char inuse;	// Is this entry in use or not?
}file_action;

typedef struct posix_spawn_file_actions_t
{
	int nfileactions;
	file_action fa[POSIX_SPAWN_NFACTIONS];
}posix_spawn_file_actions_t;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int posix_spawn (pid_t*, const char*, const posix_spawn_file_actions_t*,
	const posix_spawnattr_t*, char **, char **);

int posix_spawnattr_init (posix_spawnattr_t *attr);
int posix_spawnattr_setflags (posix_spawnattr_t *attr, int flags);
int posix_spawnattr_setpgroup (posix_spawnattr_t *attr, int pgroup);
int posix_spawnattr_destroy (posix_spawnattr_t *attr);

int posix_spawn_file_actions_init (posix_spawn_file_actions_t *factions);
int posix_spawn_file_actions_addopen (posix_spawn_file_actions_t *factions, int fildes, char *path, int oflag, int mode);
int posix_spawn_file_actions_adddup2 (posix_spawn_file_actions_t *factions, int fildes, int newfildes);
int posix_spawn_file_actions_destroy (posix_spawn_file_actions_t *factions);

#endif
