/*
 * Copyright (c) 2018, Erlend Sveen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SYS_MMAN_H
#define SYS_MMAN_H

// https://linux.die.net/man/2/mmap
// https://pubs.opengroup.org/onlinepubs/007904875/basedefs/sys/mman.h.html
// https://pubs.opengroup.org/onlinepubs/007904875/functions/mmap.html

///////////////////////////////////////////////////////////////////////////////
// Include
///////////////////////////////////////////////////////////////////////////////
#include <sys/types.h>

///////////////////////////////////////////////////////////////////////////////
// Definitions
///////////////////////////////////////////////////////////////////////////////
#define PROT_READ 1
#define PROT_WRITE 2
#define PROT_EXEC 4
#define PROT_NONE 0

#define MAP_SHARED (1 << 0)
#define MAP_PRIVATE (1 << 1)
#define MAP_FIXED (1 << 2)
#define MAP_ANONYMOUS (1 << 3)

#define MAP_FAILED ((void*) -1)

///////////////////////////////////////////////////////////////////////////////
// Structures
///////////////////////////////////////////////////////////////////////////////
struct posix_typed_mem_info
{
	size_t posix_tmi_length;
};

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int mlock (const void*, size_t);
int mlockall (int);
void *mmap (void*, size_t, int, int, int, off_t);
int mprotect (void*, size_t, int);
int msync (void*, size_t, int);
int munlock (const void*, size_t);
int munlockall (void);
int munmap (void*, size_t);
int posix_madvise (void*, size_t, int);
int posix_mem_offset (const void *restrict, size_t, off_t *restrict,
	size_t *restrict, int *restrict);
int posix_typed_mem_get_info (int, struct posix_typed_mem_info*);
int posix_typed_mem_open (const char*, int, int);
int shm_open (const char*, int, mode_t);
int shm_unlink (const char*);

#endif
