/*
 * Copyright (c) 2018, Erlend Sveen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef STATVFS_H
#define STATVFS_H

///////////////////////////////////////////////////////////////////////////////
// Include the types used by stat, as required by POSIX.
///////////////////////////////////////////////////////////////////////////////
#include <sys/types.h>

///////////////////////////////////////////////////////////////////////////////
// Constants used by stat, required by POSIX
///////////////////////////////////////////////////////////////////////////////
#define ST_RDONLY 0
#define ST_NOSUID 1

///////////////////////////////////////////////////////////////////////////////
// Stat of a virtual file system, with POSIX compatible names.
///////////////////////////////////////////////////////////////////////////////
struct statvfs
{
	unsigned long f_bsize;		// File system block size
	unsigned long f_frsize;		// Fragment size

	fsblkcnt_t f_blocks;		// Size of fs in f_frsize units
	fsblkcnt_t f_bfree;			// Number of free blocks
	fsblkcnt_t f_bavail;		// Number of free blocks for unprivileged users

	fsfilcnt_t f_files;			// Number of nodes
	fsfilcnt_t f_ffree;			// Number of free nodes
	fsfilcnt_t f_favail;		// Number of free nodes for unprivileged users

	unsigned long f_fsid;		// Filesystem ID
	unsigned long f_flag;		// Mount flags
	unsigned long f_namemax;	// Maximum filename length
};

///////////////////////////////////////////////////////////////////////////////
// Function declarations required by POSIX.
///////////////////////////////////////////////////////////////////////////////
int fstatvfs (int, struct statvfs *);
int statvfs (const char *restrict, struct statvfs *restrict);

#endif
