/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include "lcdtest.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
int initbus (lcd *l)
{
	l->datafd = open ("/dev/spi3", O_RDWR);

	if (l->datafd < 0)
	{
		printf ("Failed to open spi3: %s\n", strerror (errno));
		return 1;
	}

	l->dcfd = open ("/dev/spi1", O_RDWR);

	if (l->dcfd < 0)
	{
		printf ("Failed to open spi1: %s\n", strerror (errno));
		return 1;
	}

	if (ioctl (l->dcfd, SPICS, 0))
		printf ("DC failed\n");

	if (ioctl (l->datafd, SPICS, 0))
		printf ("CS failed\n");

	return 0;
}

int cslow (lcd *l)
{
	if (ioctl (l->datafd, SPICS, 3))
		printf ("CS low failed\n");
	return 0;
}

int cshigh (lcd *l)
{
	if (ioctl (l->datafd, SPICS, 0))
		printf ("CS Disable wailed with: %s\n", strerror (errno));
	return 0;
}

int writecommand (lcd *l, unsigned char command)
{
	if (ioctl (l->dcfd, SPICS, 2))
		printf ("Command low failed\n");

	writedata (l, &command, 1);

	if (ioctl (l->dcfd, SPICS, 0))
		printf ("Command high failed\n");

	return 0;
}

int writedata (lcd *l, unsigned char *data, int nbytes)
{
	int nwrite = write (l->datafd, data, nbytes);

	if (nwrite < 0)
		printf ("Write failed with: %s\n", strerror (errno));
	else
	{
		// Wait for completion
		int timer = 1000000;

		while (timer && ioctl (l->datafd, SPIBUSY, 0))
			timer--;

		// Check
		if (!timer)
			printf ("Timeout waiting for data\n");
	}

	return 0;
}

int writecommandarg0 (lcd *l, unsigned char cmd)
{
	cslow (l);
	writecommand (l, cmd);
	cshigh (l);
	return 0;
}

int writecommandarg1 (lcd *l, unsigned char cmd, unsigned char a1)
{
	unsigned char args[] = {a1};

	cslow (l);
	writecommand (l, cmd);
	writedata (l, args, sizeof(args));
	cshigh (l);
	return 0;
}

int writecommandarg2 (lcd *l, unsigned char cmd, unsigned char a1, unsigned char a2)
{
	unsigned char args[] = {a1, a2};

	cslow (l);
	writecommand (l, cmd);
	writedata (l, args, sizeof(args));
	cshigh (l);
	return 0;
}

int writecommandarg3 (lcd *l, unsigned char cmd, unsigned char a1, unsigned char a2, unsigned char a3)
{
	unsigned char args[] = {a1, a2, a3};

	cslow (l);
	writecommand (l, cmd);
	writedata (l, args, sizeof(args));
	cshigh (l);
	return 0;
}

int writecommandarg4 (lcd *l, unsigned char cmd,
	unsigned char a1, unsigned char a2, unsigned char a3, unsigned char a4)
{
	unsigned char args[] = {a1, a2, a3, a4};

	cslow (l);
	writecommand (l, cmd);
	writedata (l, args, sizeof(args));
	cshigh (l);
	return 0;
}

int writecommandarg5 (lcd *l, unsigned char cmd,
	unsigned char a1, unsigned char a2, unsigned char a3, unsigned char a4, unsigned char a5)
{
	unsigned char args[] = {a1, a2, a3, a4, a5};

	cslow (l);
	writecommand (l, cmd);
	writedata (l, args, sizeof(args));
	cshigh (l);
	return 0;
}
