/*
 * Copyright (C) 2018 Erlend Sveen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <sys/ioctl.h>

struct pwmControlPacket
{
	unsigned short period[3];
	unsigned short cmp[3];
	unsigned char pinvalues[4];
};

void Delay (unsigned int time)
{
	struct timespec req;

	req.tv_sec = time / 1000;
	req.tv_nsec = (time % 1000) * 1000000;

	nanosleep (&req, 0);
}

void Channel (struct pwmControlPacket *c, char *dir, int i)
{
	if (dir[i])
	{
		if (c->cmp[i] < c->period[i])
			c->cmp[i]++;
		else
		{
			c->pinvalues[i] = 0;
			dir[i] = 0;
		}
	}
	else
	{
		if (c->cmp[i] > 0)
			c->cmp[i]--;
		else
		{
			c->pinvalues[i] = 1;
			dir[i] = 1;
		}
	}
}

int main (int argc, char **argv)
{
	if (argc < 2)
	{
		printf ("Please specify device\n");
		return 1;
	}

	int fd = open (argv[1], O_RDWR);

	if (fd < 0)
	{
		printf ("Error opening device: %s\n", strerror (errno));
		return 2;
	}

	struct pwmControlPacket ctrl;
	char dir[3];

	memset (&ctrl, 0, sizeof (ctrl));
	dir[0] = 0;
	dir[1] = 1;
	dir[2] = 0;
	ctrl.period[0] = 2000;
	ctrl.period[1] = 2000;
	ctrl.period[2] = 2000;
	ctrl.cmp[0] = 0;
	ctrl.cmp[1] = 2000 / 3;
	ctrl.cmp[2] = (2000 / 3) * 2;
	ctrl.pinvalues[0] = 0;
	ctrl.pinvalues[1] = 1;
	ctrl.pinvalues[2] = 0;
	ctrl.pinvalues[3] = 1;

	while (1)
	{
		int w = write (fd, &ctrl, sizeof (ctrl));

		if (w != sizeof (ctrl))
		{
			printf ("Write failed with: %s\n", strerror (errno));
			break;
		}

		Channel (&ctrl, dir, 0);
		Channel (&ctrl, dir, 1);
		Channel (&ctrl, dir, 2);
		Delay (1);
	}

	close (fd);
	return 0;
}
