/*
 * Copyright (c) 2018, Erlend Sveen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

///////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "structures.h"

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
struct function *newfunc (char *name, char *address)
{
	struct function *f = malloc (sizeof (struct function));

	if (!f)
		return 0;

	memset (f, 0, sizeof (struct function));
	f->name = malloc (strlen (name) + 1);

	if (f->name)
		strcpy (f->name, name);

	f->address = strtol (address, 0, 16);

	//printf ("Created function object for \"%s\", address %.8X\n",
	//	f->name, f->address);

	return f;
}

struct instruction *newinst (char *address, char *rest)
{
	struct instruction *inst = malloc (sizeof (struct instruction));

	if (!inst)
		return 0;

	memset (inst, 0, sizeof (struct instruction));
	inst->address = strtol (address, 0, 16);

	char *next = strtok (rest, "\t");

	if (next)
	{
		if (next[4] == ' ')
		{
			for (int i = 4; i < 4+5; i++)
				next[i] = next[i+1];
		}

		inst->data = strtol (next, 0, 16);
	}

	next = strtok (0, "\t");

	if (next)
	{
		inst->operation = malloc (strlen (next) + 1);
		if (inst->operation)
			strcpy (inst->operation, next);
	}

	next = strtok (0, "");

	if (next)
	{
		inst->rest = malloc (strlen (next) + 1);
		if (inst->rest)
			strcpy (inst->rest, next);
	}

	if (!inst->operation || !inst->rest)
	{
		if (inst->operation)
			free (inst->operation);
		if (inst->rest)
			free (inst->rest);
		free (inst);
		return 0;
	}

	//printf ("Created instruction address %.8X, data %.8X instruction %s rest %s\n",
	//	inst->address, inst->data, inst->operation, inst->rest);

	return inst;
}
